# Licensed under a 3-clause BSD style license - see LICENSE.rst
"""
Tests models.parameters
"""
# pylint: disable=invalid-name

import functools
import itertools
import unittest.mock as mk

import numpy as np
import pytest

from astropy import units as u
from astropy.modeling import fitting, models
from astropy.modeling.core import FittableModel, Model
from astropy.modeling.parameters import (
    InputParameterError,
    Parameter,
    _tofloat,
    param_repr_oneline,
)
from astropy.utils.data import get_pkg_data_filename

from . import irafutil


def setter1(val):
    return val


def setter2(val, model):
    model.do_something(val)
    return val * model.p


def getter1(val):
    return val


class SetterModel(FittableModel):
    n_inputs = 2
    n_outputs = 1

    xc = Parameter(default=1, setter=setter1, getter=getter1)
    yc = Parameter(default=1, setter=setter2, getter=getter1)

    def do_something(self, v):
        pass

    def __init__(self, xc, yc, p):
        self.p = p  # p is a value intended to be used by the setter
        super().__init__()
        self.xc = xc
        self.yc = yc

    def evaluate(self, x, y, xc, yc):
        return (x - xc) ** 2 + (y - yc) ** 2


class TParModel(Model):
    """
    A toy model to test parameters machinery
    """

    coeff = Parameter()
    e = Parameter()

    def __init__(self, coeff, e, **kwargs):
        super().__init__(coeff=coeff, e=e, **kwargs)

    @staticmethod
    def evaluate(coeff, e):
        pass


class MockModel(FittableModel):
    alpha = Parameter(name="alpha", default=42)

    @staticmethod
    def evaluate(*args):
        pass


def test__tofloat():
    # iterable
    value = _tofloat([1, 2, 3])
    assert isinstance(value, np.ndarray)
    assert (value == np.array([1, 2, 3])).all()
    assert np.all([isinstance(val, float) for val in value])
    value = _tofloat(np.array([1, 2, 3]))
    assert isinstance(value, np.ndarray)
    assert (value == np.array([1, 2, 3])).all()
    assert np.all([isinstance(val, float) for val in value])
    MESSAGE = r"Parameter of .* could not be converted to float"
    with pytest.raises(InputParameterError, match=MESSAGE):
        _tofloat("test")

    # quantity
    assert _tofloat(1 * u.m) == 1 * u.m

    # dimensions/scalar array
    value = _tofloat(np.asanyarray(3))
    assert isinstance(value, float)
    assert value == 3

    # A regular number
    value = _tofloat(3)
    assert isinstance(value, float)
    assert value == 3
    value = _tofloat(3.0)
    assert isinstance(value, float)
    assert value == 3
    value = _tofloat(np.float32(3))
    assert isinstance(value, float)
    assert value == 3
    value = _tofloat(np.float64(3))
    assert isinstance(value, float)
    assert value == 3
    value = _tofloat(np.int32(3))
    assert isinstance(value, float)
    assert value == 3
    value = _tofloat(np.int64(3))
    assert isinstance(value, float)
    assert value == 3

    # boolean
    MESSAGE = r"Expected parameter to be of numerical type, not boolean"
    with pytest.raises(InputParameterError, match=MESSAGE):
        _tofloat(True)
    with pytest.raises(InputParameterError, match=MESSAGE):
        _tofloat(False)

    # other
    class Value:
        pass

    MESSAGE = r"Don't know how to convert parameter of .* to float"
    with pytest.raises(InputParameterError, match=MESSAGE):
        _tofloat(Value)


def test_parameter_properties():
    """Test if getting / setting of Parameter properties works."""

    p = Parameter("alpha", default=1)

    assert p.name == "alpha"

    # Parameter names are immutable
    with pytest.raises(AttributeError):
        p.name = "beta"

    assert p.fixed is False
    p.fixed = True
    assert p.fixed is True

    assert p.tied is False
    p.tied = lambda _: 0

    p.tied = False
    assert p.tied is False

    assert p.min is None
    p.min = 42
    assert p.min == 42
    p.min = None
    assert p.min is None

    assert p.max is None
    p.max = 41
    assert p.max == 41


def test_parameter_operators():
    """Test if the parameter arithmetic operators work."""

    par = Parameter("alpha", default=42)
    num = 42.0
    val = 3

    assert par - val == num - val
    assert val - par == val - num
    assert par / val == num / val
    assert val / par == val / num
    assert par**val == num**val
    assert val**par == val**num
    assert par < 45
    assert par > 41
    assert par <= par
    assert par >= par
    assert par == par
    assert -par == -num
    assert abs(par) == abs(num)


# Test inherited models


class M1(Model):
    m1a = Parameter(default=1.0)
    m1b = Parameter(default=5.0)

    def evaluate():
        pass


class M2(M1):
    m2c = Parameter(default=11.0)


class M3(M2):
    m3d = Parameter(default=20.0)


def test_parameter_inheritance():
    mod = M3()
    assert mod.m1a == 1.0
    assert mod.m1b == 5.0
    assert mod.m2c == 11.0
    assert mod.m3d == 20.0
    for key in ["m1a", "m1b", "m2c", "m3d"]:
        assert key in mod.__dict__
    assert mod.param_names == ("m1a", "m1b", "m2c", "m3d")


def test_param_metric():
    mod = M3()
    assert mod._param_metrics["m1a"]["slice"] == slice(0, 1)
    assert mod._param_metrics["m1b"]["slice"] == slice(1, 2)
    assert mod._param_metrics["m2c"]["slice"] == slice(2, 3)
    assert mod._param_metrics["m3d"]["slice"] == slice(3, 4)
    mod._parameters_to_array()
    assert (mod._parameters == np.array([1.0, 5.0, 11.0, 20], dtype=np.float64)).all()


class TestParameters:
    def setup_class(self):
        """
        Unit tests for parameters

        Read an iraf database file created by onedspec.identify.  Use the
        information to create a 1D Chebyshev model and perform the same fit.

        Create also a gaussian model.
        """
        test_file = get_pkg_data_filename("data/idcompspec.fits")
        f = open(test_file)
        lines = f.read()
        reclist = lines.split("begin")
        f.close()
        record = irafutil.IdentifyRecord(reclist[1])
        self.icoeff = record.coeff
        order = int(record.fields["order"])
        self.model = models.Chebyshev1D(order - 1)
        self.gmodel = models.Gaussian1D(2, mean=3, stddev=4)
        self.linear_fitter = fitting.LinearLSQFitter()
        self.x = record.x
        self.y = record.z
        self.yy = np.array([record.z, record.z])

    def test_set_parameters_as_list(self):
        """Tests updating parameters using a list."""

        self.model.parameters = [30, 40, 50, 60, 70]
        assert (self.model.parameters == [30.0, 40.0, 50.0, 60, 70]).all()

    def test_set_parameters_as_array(self):
        """Tests updating parameters using an array."""

        self.model.parameters = np.array([3, 4, 5, 6, 7])
        assert (self.model.parameters == [3.0, 4.0, 5.0, 6.0, 7.0]).all()

    def test_set_as_tuple(self):
        """Tests updating parameters using a tuple."""

        self.model.parameters = (1, 2, 3, 4, 5)
        assert (self.model.parameters == [1, 2, 3, 4, 5]).all()

    def test_set_model_attr_seq(self):
        """
        Tests updating the parameters attribute when a model's
        parameter (in this case coeff) is updated.
        """

        self.model.parameters = [0, 0.0, 0.0, 0, 0]
        self.model.c0 = 7
        assert (self.model.parameters == [7, 0.0, 0.0, 0, 0]).all()

    def test_set_model_attr_num(self):
        """Update the parameter list when a model's parameter is updated."""

        self.gmodel.amplitude = 7
        assert (self.gmodel.parameters == [7, 3, 4]).all()

    def test_set_item(self):
        """Update the parameters using indexing."""

        self.model.parameters = [1, 2, 3, 4, 5]
        tpar = self.model.parameters
        tpar[0] = 10.0
        self.model.parameters = tpar
        assert (self.model.parameters == [10, 2, 3, 4, 5]).all()
        assert self.model.c0 == 10

    def test_wrong_size1(self):
        """
        Tests raising an error when attempting to reset the parameters
        using a list of a different size.
        """

        MESSAGE = (
            r"Input parameter values not compatible with the model parameters array: .*"
        )
        with pytest.raises(InputParameterError, match=MESSAGE):
            self.model.parameters = [1, 2, 3]

    def test_wrong_size2(self):
        """
        Tests raising an exception when attempting to update a model's
        parameter (in this case coeff) with a sequence of the wrong size.
        """

        MESSAGE = (
            r"Value for parameter c0 does not match shape or size\nexpected by model .*"
            r" vs .*"
        )
        with pytest.raises(InputParameterError, match=MESSAGE):
            self.model.c0 = [1, 2, 3]

    def test_wrong_shape(self):
        """
        Tests raising an exception when attempting to update a model's
        parameter and the new value has the wrong shape.
        """

        MESSAGE = (
            r"Value for parameter amplitude does not match shape or size\nexpected by"
            r" model .* vs .*"
        )
        with pytest.raises(InputParameterError, match=MESSAGE):
            self.gmodel.amplitude = [1, 2]

    def test_par_against_iraf(self):
        """
        Test the fitter modifies model.parameters.

        Uses an iraf example.
        """

        new_model = self.linear_fitter(self.model, self.x, self.y)
        np.testing.assert_allclose(
            new_model.parameters,
            np.array(
                [
                    4826.1066602783685,
                    952.8943813407858,
                    12.641236013982386,
                    -1.7910672553339604,
                    0.90252884366711317,
                ]
            ),
            rtol=10 ** (-2),
        )

    def testPolynomial1D(self):
        d = {"c0": 11, "c1": 12, "c2": 13, "c3": 14}
        p1 = models.Polynomial1D(3, **d)
        np.testing.assert_equal(p1.parameters, [11, 12, 13, 14])

    def test_poly1d_multiple_sets(self):
        p1 = models.Polynomial1D(3, n_models=3)
        np.testing.assert_equal(
            p1.parameters, [0.0, 0.0, 0.0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        )
        np.testing.assert_array_equal(p1.c0, [0, 0, 0])
        p1.c0 = [10, 10, 10]
        np.testing.assert_equal(
            p1.parameters, [10.0, 10.0, 10.0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        )

    def test_par_slicing(self):
        """
        Test assigning to a parameter slice
        """
        p1 = models.Polynomial1D(3, n_models=3)
        p1.c0[:2] = [10, 10]
        np.testing.assert_equal(
            p1.parameters, [10.0, 10.0, 0.0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        )

    def test_poly2d(self):
        p2 = models.Polynomial2D(degree=3)
        p2.c0_0 = 5
        np.testing.assert_equal(p2.parameters, [5, 0, 0, 0, 0, 0, 0, 0, 0, 0])

    def test_poly2d_multiple_sets(self):
        kw = {
            "c0_0": [2, 3],
            "c1_0": [1, 2],
            "c2_0": [4, 5],
            "c0_1": [1, 1],
            "c0_2": [2, 2],
            "c1_1": [5, 5],
        }
        p2 = models.Polynomial2D(2, **kw)
        np.testing.assert_equal(p2.parameters, [2, 3, 1, 2, 4, 5, 1, 1, 2, 2, 5, 5])

    def test_shift_model_parameters1d(self):
        sh1 = models.Shift(2)
        sh1.offset = 3
        assert sh1.offset == 3
        assert sh1.offset.value == 3

    def test_scale_model_parametersnd(self):
        sc1 = models.Scale([2, 2])
        sc1.factor = [3, 3]
        assert np.all(sc1.factor == [3, 3])
        np.testing.assert_array_equal(sc1.factor.value, [3, 3])

    def test_bounds(self):
        # Valid __init__
        param = Parameter(bounds=(1, 2))
        assert param.bounds == (1, 2)
        param = Parameter(min=1, max=2)
        assert param.bounds == (1, 2)

        # Errors __init__
        MESSAGE = r"bounds may not be specified simultaneously with min or max .*"
        with pytest.raises(ValueError, match=MESSAGE):
            Parameter(bounds=(1, 2), min=1, name="test")
        with pytest.raises(ValueError, match=MESSAGE):
            Parameter(bounds=(1, 2), max=2, name="test")
        with pytest.raises(ValueError, match=MESSAGE):
            Parameter(bounds=(1, 2), min=1, max=2, name="test")

        # Setters
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param.bounds == (None, None) == param._bounds

        # Set errors
        MESSAGE = "{} value must be a number or a Quantity"
        with pytest.raises(TypeError, match=MESSAGE.format("Min")):
            param.bounds = ("test", None)
        with pytest.raises(TypeError, match=MESSAGE.format("Max")):
            param.bounds = (None, "test")

        # Set number
        param.bounds = (1, 2)
        assert param.bounds == (1, 2) == param._bounds

        # Set Quantity
        param.bounds = (1 * u.m, 2 * u.m)
        assert param.bounds == (1, 2) == param._bounds

    def test_modify_value(self):
        param = Parameter(name="test", default=[1, 2, 3])
        assert (param.value == [1, 2, 3]).all()

        # Errors
        MESSAGE = r"Slice assignment outside the parameter dimensions for 'test'"
        with pytest.raises(InputParameterError, match=MESSAGE):
            param[slice(0, 0)] = 2

        MESSAGE = r"Input dimension 3 invalid for 'test' parameter with dimension 1"
        with pytest.raises(InputParameterError, match=MESSAGE):
            param[3] = np.array([5])

        # assignment of a slice
        param[slice(0, 2)] = [4, 5]
        assert (param.value == [4, 5, 3]).all()

        # assignment of a value
        param[2] = 6
        assert (param.value == [4, 5, 6]).all()

    def test__set_unit(self):
        param = Parameter(name="test", default=[1, 2, 3])
        assert param.unit is None

        # No force Error (no existing unit)
        MESSAGE = r"Cannot attach units to parameters that were .*"
        with pytest.raises(ValueError, match=MESSAGE):
            param._set_unit(u.m)

        # Force
        param._set_unit(u.m, True)
        assert param.unit == u.m

        # Force magnitude unit (mag=False)
        MESSAGE = r"This parameter does not support the magnitude units such as .*"
        with pytest.raises(ValueError, match=MESSAGE):
            param._set_unit(u.ABmag, True)

        # Force magnitude unit (mag=True)
        param._mag = True
        param._set_unit(u.ABmag, True)
        assert param._unit == u.ABmag

        # No force Error (existing unit)
        MESSAGE = r"Cannot change the unit attribute directly, instead change the .*"
        with pytest.raises(ValueError, match=MESSAGE):
            param._set_unit(u.K)

    def test_quantity(self):
        param = Parameter(name="test", default=[1, 2, 3])
        assert param.unit is None
        assert param.quantity is None

        param = Parameter(name="test", default=[1, 2, 3], unit=u.m)
        assert param.unit == u.m
        assert (param.quantity == np.array([1, 2, 3]) * u.m).all()

    def test_shape(self):
        # Array like
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param.shape == (4,)
        # Reshape error
        MESSAGE = r"cannot reshape array of size 4 into shape .*"
        with pytest.raises(ValueError, match=MESSAGE):
            param.shape = (5,)
        # Reshape success
        param.shape = (2, 2)
        assert param.shape == (2, 2)
        assert (param.value == [[1, 2], [3, 4]]).all()

        # Scalar
        param = Parameter(name="test", default=1)
        assert param.shape == ()
        # Reshape error
        MESSAGE = r"Cannot assign this shape to a scalar quantity"
        with pytest.raises(ValueError, match=MESSAGE):
            param.shape = (5,)
        param.shape = (1,)

        # single value
        param = Parameter(name="test", default=np.array([1]))
        assert param.shape == (1,)
        # Reshape error
        with pytest.raises(ValueError, match=MESSAGE):
            param.shape = (5,)
        param.shape = ()

    def test_size(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param.size == 4

        param = Parameter(name="test", default=[1])
        assert param.size == 1

        param = Parameter(name="test", default=1)
        assert param.size == 1

    def test_std(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param.std is None
        assert param._std is None

        param.std = 5
        assert param.std == 5 == param._std

    def test_fixed(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param.fixed is False
        assert param._fixed is False

        # Set error
        MESSAGE = r"Value must be boolean"
        with pytest.raises(ValueError, match=MESSAGE):
            param.fixed = 3

        # Set
        param.fixed = True
        assert param.fixed is True
        assert param._fixed is True

    def test_tied(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param.tied is False
        assert param._tied is False

        # Set error
        MESSAGE = r"Tied must be a callable or set to False or None"
        with pytest.raises(TypeError, match=MESSAGE):
            param.tied = mk.NonCallableMagicMock()

        # Set None
        param.tied = None
        assert param.tied is None
        assert param._tied is None

        # Set False
        param.tied = False
        assert param.tied is False
        assert param._tied is False

        # Set other
        tied = mk.MagicMock()
        param.tied = tied
        assert param.tied == tied == param._tied

    def test_validator(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param._validator is None

        valid = mk.MagicMock()
        param.validator(valid)
        assert param._validator == valid

        MESSAGE = r"This decorator method expects a callable.*"
        with pytest.raises(ValueError, match=MESSAGE):
            param.validator(mk.NonCallableMagicMock())

    def test_validate(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param._validator is None
        assert param.model is None

        # Run without validator
        param.validate(mk.MagicMock())

        # Run with validator but no Model
        validator = mk.MagicMock()
        param.validator(validator)
        assert param._validator == validator
        param.validate(mk.MagicMock())
        assert validator.call_args_list == []

        # Full validate
        param._model = mk.MagicMock()
        value = mk.MagicMock()
        param.validate(value)
        assert validator.call_args_list == [mk.call(param._model, value)]

    def test_copy(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])
        copy_param = param.copy()

        assert (param == copy_param).all()
        assert id(param) != id(copy_param)

    def test_model(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param.model is None
        assert param._model is None
        assert param._model_required is False
        assert (param._value == [1, 2, 3, 4]).all()

        setter = mk.MagicMock()
        getter = mk.MagicMock()
        param._setter = setter
        param._getter = getter

        # No Model Required
        param._value = [5, 6, 7, 8]
        model0 = mk.MagicMock()
        setter0 = mk.MagicMock()
        getter0 = mk.MagicMock()
        with mk.patch.object(
            Parameter, "_create_value_wrapper", side_effect=[setter0, getter0]
        ) as mkCreate:
            param.model = model0
            assert param.model == model0 == param._model
            assert param._setter == setter0
            assert param._getter == getter0
            assert mkCreate.call_args_list == [
                mk.call(setter, model0),
                mk.call(getter, model0),
            ]
            assert param._value == [5, 6, 7, 8]

        param._setter = setter
        param._getter = getter

        # Model required
        param._model_required = True
        model1 = mk.MagicMock()
        setter1 = mk.MagicMock()
        getter1 = mk.MagicMock()
        setter1.return_value = np.array([9, 10, 11, 12])
        getter1.return_value = np.array([9, 10, 11, 12])
        with mk.patch.object(
            Parameter, "_create_value_wrapper", side_effect=[setter1, getter1]
        ) as mkCreate:
            param.model = model1
            assert param.model == model1 == param._model
            assert param._setter == setter1
            assert param._getter == getter1
            assert mkCreate.call_args_list == [
                mk.call(setter, model1),
                mk.call(getter, model1),
            ]
            assert (param.value == [9, 10, 11, 12]).all()

        param._setter = setter
        param._getter = getter
        param._default = None
        with mk.patch.object(
            Parameter, "_create_value_wrapper", side_effect=[setter1, getter1]
        ) as mkCreate:
            param.model = model1
            assert param.model == model1 == param._model
            assert param._setter == setter1
            assert param._getter == getter1
            assert mkCreate.call_args_list == [
                mk.call(setter, model1),
                mk.call(getter, model1),
            ]
            assert param._value is None

    def test_value(self):
        param = Parameter(name="test", default=1)
        assert not isinstance(param.value, np.ndarray)
        assert param.value == 1

        param = Parameter(name="test", default=[1])
        assert not isinstance(param.value, np.ndarray)
        assert param.value == 1

        param = Parameter(name="test", default=[[1]])
        assert not isinstance(param.value, np.ndarray)
        assert param.value == 1

        param = Parameter(name="test", default=np.array([1]))
        assert not isinstance(param.value, np.ndarray)
        assert param.value == 1

        param = Parameter(name="test", default=[1, 2, 3])
        assert isinstance(param.value, np.ndarray)
        assert (param.value == [1, 2, 3]).all()

        param = Parameter(name="test", default=[1], setter=setter1, getter=getter1)
        assert not isinstance(param.value, np.ndarray)
        assert param.value == 1

        param = Parameter(name="test", default=[[1]], setter=setter1, getter=getter1)
        assert not isinstance(param.value, np.ndarray)
        assert param.value == 1

        param = Parameter(
            name="test", default=np.array([1]), setter=setter1, getter=getter1
        )
        assert not isinstance(param.value, np.ndarray)
        assert param.value == 1

    def test_raw_value(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])

        # Normal case
        assert (param._raw_value == param.value).all()

        # Bad setter
        param._setter = True
        param._internal_value = 4
        assert param._raw_value == 4

    def test__create_value_wrapper(self):
        param = Parameter(name="test", default=[1, 2, 3, 4])

        # Bad ufunc
        MESSAGE = r"A numpy.ufunc used for Parameter getter/setter .*"
        with pytest.raises(TypeError, match=MESSAGE):
            param._create_value_wrapper(np.add, mk.MagicMock())
        # Good ufunc
        with mk.patch(
            "astropy.modeling.parameters._wrap_ufunc", autospec=True
        ) as mkWrap:
            assert (
                param._create_value_wrapper(np.negative, mk.MagicMock())
                == mkWrap.return_value
            )
            assert mkWrap.call_args_list == [mk.call(np.negative)]

        # None
        assert param._create_value_wrapper(None, mk.MagicMock()) is None

        # wrapper with one argument
        def wrapper1(a):
            pass

        assert param._create_value_wrapper(wrapper1, mk.MagicMock()) == wrapper1

        # wrapper with two argument2
        def wrapper2(a, b):
            pass

        # model is None
        assert param._model_required is False
        assert param._create_value_wrapper(wrapper2, None) == wrapper2
        assert param._model_required is True
        # model is not None
        param._model_required = False
        model = mk.MagicMock()
        partial_wrapper = param._create_value_wrapper(wrapper2, model)
        assert isinstance(partial_wrapper, functools.partial)
        assert partial_wrapper.func is wrapper2
        assert partial_wrapper.args == ()
        assert list(partial_wrapper.keywords.keys()) == ["b"]
        assert partial_wrapper.keywords["b"] is model

        # wrapper with more than 2 arguments
        def wrapper3(a, b, c):
            pass

        MESSAGE = r"Parameter getter/setter must be a function .*"
        with pytest.raises(TypeError, match=MESSAGE):
            param._create_value_wrapper(wrapper3, mk.MagicMock())

    def test_bool(self):
        # single value is true
        param = Parameter(name="test", default=1)
        assert param.value == 1
        assert np.all(param)
        assert param

        # single value is false
        param = Parameter(name="test", default=0)
        assert param.value == 0
        assert not np.all(param)
        assert not param

        # vector value all true
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert np.all(param.value == [1, 2, 3, 4])
        assert np.all(param)
        assert param

        # vector value at least one false
        param = Parameter(name="test", default=[1, 2, 0, 3, 4])
        assert np.all(param.value == [1, 2, 0, 3, 4])
        assert not np.all(param)
        assert not param

    def test_param_repr_oneline(self):
        # Single value no units
        param = Parameter(name="test", default=1)
        assert param_repr_oneline(param) == "1."

        # Vector value no units
        param = Parameter(name="test", default=[1, 2, 3, 4])
        assert param_repr_oneline(param) == "[1., 2., 3., 4.]"

        # Single value units
        param = Parameter(name="test", default=1 * u.m)
        assert param_repr_oneline(param) == "1. m"

        # Vector value units
        param = Parameter(name="test", default=[1, 2, 3, 4] * u.m)
        assert param_repr_oneline(param) == "[1., 2., 3., 4.] m"

    def test_getter_setter(self):
        msg = "setter and getter must both be input"
        with pytest.raises(ValueError, match=msg):
            Parameter(name="test", default=1, getter=getter1)
        with pytest.raises(ValueError, match=msg):
            Parameter(name="test", default=1, setter=setter1)


class TestMultipleParameterSets:
    def setup_class(self):
        self.x1 = np.arange(1, 10, 0.1)
        self.y, self.x = np.mgrid[:10, :7]
        self.x11 = np.array([self.x1, self.x1]).T
        self.gmodel = models.Gaussian1D(
            [12, 10], [3.5, 5.2], stddev=[0.4, 0.7], n_models=2
        )

    def test_change_par(self):
        """
        Test that a change to one parameter as a set propagates to param_sets.
        """
        self.gmodel.amplitude = [1, 10]
        np.testing.assert_almost_equal(
            self.gmodel.param_sets,
            np.array(
                [
                    [1.0, 10],
                    [3.5, 5.2],
                    [0.4, 0.7],
                ]
            ),
        )
        np.all(self.gmodel.parameters == [1.0, 10.0, 3.5, 5.2, 0.4, 0.7])

    def test_change_par2(self):
        """
        Test that a change to one single parameter in a set propagates to
        param_sets.
        """
        self.gmodel.amplitude[0] = 11
        np.testing.assert_almost_equal(
            self.gmodel.param_sets,
            np.array(
                [
                    [11.0, 10],
                    [3.5, 5.2],
                    [0.4, 0.7],
                ]
            ),
        )
        np.all(self.gmodel.parameters == [11.0, 10.0, 3.5, 5.2, 0.4, 0.7])

    def test_change_parameters(self):
        self.gmodel.parameters = [13, 10, 9, 5.2, 0.4, 0.7]
        np.testing.assert_almost_equal(self.gmodel.amplitude.value, [13.0, 10.0])
        np.testing.assert_almost_equal(self.gmodel.mean.value, [9.0, 5.2])


class TestParameterInitialization:
    """
    This suite of tests checks most if not all cases if instantiating a model
    with parameters of different shapes/sizes and with different numbers of
    parameter sets.
    """

    def test_single_model_scalar_parameters(self):
        t = TParModel(10, 1)
        assert len(t) == 1
        assert t.model_set_axis is False
        assert np.all(t.param_sets == [[10], [1]])
        assert np.all(t.parameters == [10, 1])
        assert t.coeff.shape == ()
        assert t.e.shape == ()

    def test_single_model_scalar_and_array_parameters(self):
        t = TParModel(10, [1, 2])
        assert len(t) == 1
        assert t.model_set_axis is False
        assert np.issubdtype(t.param_sets.dtype, np.object_)
        assert len(t.param_sets) == 2
        assert np.all(t.param_sets[0] == [10])
        assert np.all(t.param_sets[1] == [[1, 2]])
        assert np.all(t.parameters == [10, 1, 2])
        assert t.coeff.shape == ()
        assert t.e.shape == (2,)

    def test_single_model_1d_array_parameters(self):
        t = TParModel([10, 20], [1, 2])
        assert len(t) == 1
        assert t.model_set_axis is False
        assert np.all(t.param_sets == [[[10, 20]], [[1, 2]]])
        assert np.all(t.parameters == [10, 20, 1, 2])
        assert t.coeff.shape == (2,)
        assert t.e.shape == (2,)

    def test_single_model_1d_array_different_length_parameters(self):
        MESSAGE = "Mismatch is between arg 0 with shape .* and arg 1 with shape .*"
        with pytest.raises(InputParameterError, match=MESSAGE):
            # Not broadcastable
            TParModel([1, 2], [3, 4, 5])

    def test_single_model_2d_array_parameters(self):
        t = TParModel([[10, 20], [30, 40]], [[1, 2], [3, 4]])
        assert len(t) == 1
        assert t.model_set_axis is False
        assert np.all(
            t.param_sets
            == [
                [[[10, 20], [30, 40]]],
                [[[1, 2], [3, 4]]],
            ]
        )
        assert np.all(t.parameters == [10, 20, 30, 40, 1, 2, 3, 4])
        assert t.coeff.shape == (2, 2)
        assert t.e.shape == (2, 2)

    def test_single_model_2d_non_square_parameters(self):
        coeff = np.array(
            [
                [10, 20],
                [30, 40],
                [50, 60],
            ]
        )
        e = np.array([[1, 2], [3, 4], [5, 6]])

        t = TParModel(coeff, e)
        assert len(t) == 1
        assert t.model_set_axis is False
        assert np.all(
            t.param_sets
            == [
                [[[10, 20], [30, 40], [50, 60]]],
                [[[1, 2], [3, 4], [5, 6]]],
            ]
        )
        assert np.all(t.parameters == [10, 20, 30, 40, 50, 60, 1, 2, 3, 4, 5, 6])
        assert t.coeff.shape == (3, 2)
        assert t.e.shape == (3, 2)

        t2 = TParModel(coeff.T, e.T)
        assert len(t2) == 1
        assert t2.model_set_axis is False
        assert np.all(
            t2.param_sets
            == [
                [[[10, 30, 50], [20, 40, 60]]],
                [[[1, 3, 5], [2, 4, 6]]],
            ]
        )
        assert np.all(t2.parameters == [10, 30, 50, 20, 40, 60, 1, 3, 5, 2, 4, 6])
        assert t2.coeff.shape == (2, 3)
        assert t2.e.shape == (2, 3)

        # Not broadcastable
        MESSAGE = "Mismatch is between arg 0 with shape .* and arg 1 with shape .*"
        with pytest.raises(InputParameterError, match=MESSAGE):
            TParModel(coeff, e.T)

        with pytest.raises(InputParameterError, match=MESSAGE):
            TParModel(coeff.T, e)

    def test_single_model_2d_broadcastable_parameters(self):
        t = TParModel([[10, 20, 30], [40, 50, 60]], [1, 2, 3])
        assert len(t) == 1
        assert t.model_set_axis is False
        assert len(t.param_sets) == 2
        assert np.issubdtype(t.param_sets.dtype, np.object_)
        assert np.all(
            t.param_sets[0]
            == [
                [[10, 20, 30], [40, 50, 60]],
            ]
        )
        assert np.all(t.param_sets[1] == [[1, 2, 3]])
        assert np.all(t.parameters == [10, 20, 30, 40, 50, 60, 1, 2, 3])

    @pytest.mark.parametrize(
        ("p1", "p2"),
        [
            (1, 2),
            (1, [2, 3]),
            ([1, 2], 3),
            ([1, 2, 3], [4, 5]),
            ([1, 2], [3, 4, 5]),
        ],
    )
    def test_two_model_incorrect_scalar_parameters(self, p1, p2):
        with pytest.raises(InputParameterError, match=r".*"):
            TParModel(p1, p2, n_models=2)

    @pytest.mark.parametrize(
        "kwargs",
        [
            {"n_models": 2},
            {"model_set_axis": 0},
            {"n_models": 2, "model_set_axis": 0},
        ],
    )
    def test_two_model_scalar_parameters(self, kwargs):
        t = TParModel([10, 20], [1, 2], **kwargs)
        assert len(t) == 2
        assert t.model_set_axis == 0
        assert np.all(t.param_sets == [[10, 20], [1, 2]])
        assert np.all(t.parameters == [10, 20, 1, 2])
        assert t.coeff.shape == (2,)
        assert t.e.shape == (2,)

    @pytest.mark.parametrize(
        "kwargs",
        [
            {"n_models": 2},
            {"model_set_axis": 0},
            {"n_models": 2, "model_set_axis": 0},
        ],
    )
    def test_two_model_scalar_and_array_parameters(self, kwargs):
        t = TParModel([10, 20], [[1, 2], [3, 4]], **kwargs)
        assert len(t) == 2
        assert t.model_set_axis == 0
        assert len(t.param_sets) == 2
        assert np.issubdtype(t.param_sets.dtype, np.object_)
        assert np.all(t.param_sets[0] == [[10], [20]])
        assert np.all(t.param_sets[1] == [[1, 2], [3, 4]])
        assert np.all(t.parameters == [10, 20, 1, 2, 3, 4])
        assert t.coeff.shape == (2,)
        assert t.e.shape == (2, 2)

    def test_two_model_1d_array_parameters(self):
        t = TParModel([[10, 20], [30, 40]], [[1, 2], [3, 4]], n_models=2)
        assert len(t) == 2
        assert t.model_set_axis == 0
        assert np.all(
            t.param_sets
            == [
                [[10, 20], [30, 40]],
                [[1, 2], [3, 4]],
            ]
        )
        assert np.all(t.parameters == [10, 20, 30, 40, 1, 2, 3, 4])
        assert t.coeff.shape == (2, 2)
        assert t.e.shape == (2, 2)

        t2 = TParModel([[10, 20, 30], [40, 50, 60]], [[1, 2, 3], [4, 5, 6]], n_models=2)
        assert len(t2) == 2
        assert t2.model_set_axis == 0
        assert np.all(
            t2.param_sets
            == [
                [[10, 20, 30], [40, 50, 60]],
                [[1, 2, 3], [4, 5, 6]],
            ]
        )
        assert np.all(t2.parameters == [10, 20, 30, 40, 50, 60, 1, 2, 3, 4, 5, 6])
        assert t2.coeff.shape == (2, 3)
        assert t2.e.shape == (2, 3)

    def test_two_model_mixed_dimension_array_parameters(self):
        MESSAGE = "Mismatch is between arg 0 with shape .* and arg 1 with shape .*"
        with pytest.raises(InputParameterError, match=MESSAGE):
            # Can't broadcast different array shapes
            TParModel(
                [[[1, 2], [3, 4]], [[5, 6], [7, 8]]],
                [[9, 10, 11], [12, 13, 14]],
                n_models=2,
            )

        t = TParModel(
            [[[10, 20], [30, 40]], [[50, 60], [70, 80]]], [[1, 2], [3, 4]], n_models=2
        )
        assert len(t) == 2
        assert t.model_set_axis == 0
        assert len(t.param_sets) == 2
        assert np.issubdtype(t.param_sets.dtype, np.object_)
        assert np.all(t.param_sets[0] == [[[10, 20], [30, 40]], [[50, 60], [70, 80]]])
        assert np.all(t.param_sets[1] == [[[1, 2]], [[3, 4]]])
        assert np.all(t.parameters == [10, 20, 30, 40, 50, 60, 70, 80, 1, 2, 3, 4])
        assert t.coeff.shape == (2, 2, 2)
        assert t.e.shape == (2, 2)

    def test_two_model_2d_array_parameters(self):
        t = TParModel(
            [[[10, 20], [30, 40]], [[50, 60], [70, 80]]],
            [[[1, 2], [3, 4]], [[5, 6], [7, 8]]],
            n_models=2,
        )
        assert len(t) == 2
        assert t.model_set_axis == 0
        assert np.all(
            t.param_sets
            == [
                [[[10, 20], [30, 40]], [[50, 60], [70, 80]]],
                [[[1, 2], [3, 4]], [[5, 6], [7, 8]]],
            ]
        )
        assert np.all(
            t.parameters == [10, 20, 30, 40, 50, 60, 70, 80, 1, 2, 3, 4, 5, 6, 7, 8]
        )
        assert t.coeff.shape == (2, 2, 2)
        assert t.e.shape == (2, 2, 2)

    def test_two_model_nonzero_model_set_axis(self):
        # An example where the model set axis is the *last* axis of the
        # parameter arrays
        coeff = np.array([[[10, 20, 30], [30, 40, 50]], [[50, 60, 70], [70, 80, 90]]])
        coeff = np.rollaxis(coeff, 0, 3)
        e = np.array([[1, 2, 3], [3, 4, 5]])
        e = np.rollaxis(e, 0, 2)
        t = TParModel(coeff, e, n_models=2, model_set_axis=-1)
        assert len(t) == 2
        assert t.model_set_axis == -1
        assert len(t.param_sets) == 2
        assert np.issubdtype(t.param_sets.dtype, np.object_)
        assert np.all(
            t.param_sets[0]
            == [
                [[10, 50], [20, 60], [30, 70]],
                [[30, 70], [40, 80], [50, 90]],
            ]
        )
        assert np.all(t.param_sets[1] == [[[1, 3], [2, 4], [3, 5]]])
        assert np.all(
            t.parameters
            == [10, 50, 20, 60, 30, 70, 30, 70, 40, 80, 50, 90, 1, 3, 2, 4, 3, 5]
        )
        assert t.coeff.shape == (2, 3, 2)  # note change in api
        assert t.e.shape == (3, 2)  # note change in api

    def test_wrong_number_of_params(self):
        MESSAGE = r"Inconsistent dimensions for parameter .* for 2 model sets.*"
        with pytest.raises(InputParameterError, match=MESSAGE):
            TParModel(coeff=[[1, 2], [3, 4]], e=(2, 3, 4), n_models=2)
        with pytest.raises(InputParameterError, match=MESSAGE):
            TParModel(coeff=[[1, 2], [3, 4]], e=(2, 3, 4), model_set_axis=0)

    def test_wrong_number_of_params2(self):
        MESSAGE = r"All parameter values must be arrays of dimension at .*"
        with pytest.raises(InputParameterError, match=MESSAGE):
            TParModel(coeff=[[1, 2], [3, 4]], e=4, n_models=2)
        with pytest.raises(InputParameterError, match=MESSAGE):
            TParModel(coeff=[[1, 2], [3, 4]], e=4, model_set_axis=0)

    def test_array_parameter1(self):
        MESSAGE = r"All parameter values must be arrays of dimension at .*"
        with pytest.raises(InputParameterError, match=MESSAGE):
            TParModel(np.array([[1, 2], [3, 4]]), 1, model_set_axis=0)

    def test_array_parameter2(self):
        MESSAGE = r"Inconsistent dimensions for parameter .* for 2 model sets.*"
        with pytest.raises(InputParameterError, match=MESSAGE):
            TParModel(np.array([[1, 2], [3, 4]]), (1, 1, 11), model_set_axis=0)

    def test_array_parameter4(self):
        """
        Test multiple parameter model with array-valued parameters of the same
        size as the number of parameter sets.
        """

        t4 = TParModel([[1, 2], [3, 4]], [5, 6], model_set_axis=False)
        assert len(t4) == 1
        assert t4.coeff.shape == (2, 2)
        assert t4.e.shape == (2,)
        assert np.issubdtype(t4.param_sets.dtype, np.object_)
        assert np.all(t4.param_sets[0] == [[1, 2], [3, 4]])
        assert np.all(t4.param_sets[1] == [5, 6])


def test_non_broadcasting_parameters():
    """
    Tests that in a model with 3 parameters that do not all mutually broadcast,
    this is determined correctly regardless of what order the parameters are
    in.
    """

    a = 3
    b = np.array([[1, 2, 3], [4, 5, 6]])
    c = np.array([[1, 2, 3, 4], [1, 2, 3, 4]])

    class TestModel(Model):
        p1 = Parameter()
        p2 = Parameter()
        p3 = Parameter()

        def evaluate(self, *args):
            return

    # a broadcasts with both b and c, but b does not broadcast with c
    MESSAGE = r"Mismatch is between arg \d with shape .* and arg \d with shape .*"
    for args in itertools.permutations((a, b, c)):
        with pytest.raises(InputParameterError, match=MESSAGE):
            TestModel(*args)


def test_setter():
    pars = np.random.rand(20).reshape((10, 2))

    model = SetterModel(xc=-1, yc=3, p=np.pi)

    for x, y in pars:
        np.testing.assert_almost_equal(model(x, y), (x + 1) ** 2 + (y - np.pi * 3) ** 2)



# This stub file was automatically generated.

__all__ = [
    "QsolMass",
    "quettaM_sun",
    "quettaMsun",
    "RsolMass",
    "ronnaM_sun",
    "ronnaMsun",
    "YsolMass",
    "yottaM_sun",
    "yottaMsun",
    "ZsolMass",
    "zettaM_sun",
    "zettaMsun",
    "EsolMass",
    "exaM_sun",
    "exaMsun",
    "PsolMass",
    "petaM_sun",
    "petaMsun",
    "TsolMass",
    "teraM_sun",
    "teraMsun",
    "GsolMass",
    "gigaM_sun",
    "gigaMsun",
    "MsolMass",
    "megaM_sun",
    "megaMsun",
    "ksolMass",
    "kiloM_sun",
    "kiloMsun",
    "hsolMass",
    "hectoM_sun",
    "hectoMsun",
    "dasolMass",
    "dekaM_sun",
    "dekaMsun",
    "decaM_sun",
    "decaMsun",
    "dsolMass",
    "deciM_sun",
    "deciMsun",
    "csolMass",
    "centiM_sun",
    "centiMsun",
    "msolMass",
    "milliM_sun",
    "milliMsun",
    "usolMass",
    "μsolMass",
    "microM_sun",
    "microMsun",
    "nsolMass",
    "nanoM_sun",
    "nanoMsun",
    "psolMass",
    "picoM_sun",
    "picoMsun",
    "fsolMass",
    "femtoM_sun",
    "femtoMsun",
    "asolMass",
    "attoM_sun",
    "attoMsun",
    "zsolMass",
    "zeptoM_sun",
    "zeptoMsun",
    "ysolMass",
    "yoctoM_sun",
    "yoctoMsun",
    "rsolMass",
    "rontoM_sun",
    "rontoMsun",
    "qsolMass",
    "quectoM_sun",
    "quectoMsun",
    "QsolRad",
    "quettaR_sun",
    "quettaRsun",
    "RsolRad",
    "ronnaR_sun",
    "ronnaRsun",
    "YsolRad",
    "yottaR_sun",
    "yottaRsun",
    "ZsolRad",
    "zettaR_sun",
    "zettaRsun",
    "EsolRad",
    "exaR_sun",
    "exaRsun",
    "PsolRad",
    "petaR_sun",
    "petaRsun",
    "TsolRad",
    "teraR_sun",
    "teraRsun",
    "GsolRad",
    "gigaR_sun",
    "gigaRsun",
    "MsolRad",
    "megaR_sun",
    "megaRsun",
    "ksolRad",
    "kiloR_sun",
    "kiloRsun",
    "hsolRad",
    "hectoR_sun",
    "hectoRsun",
    "dasolRad",
    "dekaR_sun",
    "dekaRsun",
    "decaR_sun",
    "decaRsun",
    "dsolRad",
    "deciR_sun",
    "deciRsun",
    "csolRad",
    "centiR_sun",
    "centiRsun",
    "msolRad",
    "milliR_sun",
    "milliRsun",
    "usolRad",
    "μsolRad",
    "microR_sun",
    "microRsun",
    "nsolRad",
    "nanoR_sun",
    "nanoRsun",
    "psolRad",
    "picoR_sun",
    "picoRsun",
    "fsolRad",
    "femtoR_sun",
    "femtoRsun",
    "asolRad",
    "attoR_sun",
    "attoRsun",
    "zsolRad",
    "zeptoR_sun",
    "zeptoRsun",
    "ysolRad",
    "yoctoR_sun",
    "yoctoRsun",
    "rsolRad",
    "rontoR_sun",
    "rontoRsun",
    "qsolRad",
    "quectoR_sun",
    "quectoRsun",
    "QsolLum",
    "quettaL_sun",
    "quettaLsun",
    "RsolLum",
    "ronnaL_sun",
    "ronnaLsun",
    "YsolLum",
    "yottaL_sun",
    "yottaLsun",
    "ZsolLum",
    "zettaL_sun",
    "zettaLsun",
    "EsolLum",
    "exaL_sun",
    "exaLsun",
    "PsolLum",
    "petaL_sun",
    "petaLsun",
    "TsolLum",
    "teraL_sun",
    "teraLsun",
    "GsolLum",
    "gigaL_sun",
    "gigaLsun",
    "MsolLum",
    "megaL_sun",
    "megaLsun",
    "ksolLum",
    "kiloL_sun",
    "kiloLsun",
    "hsolLum",
    "hectoL_sun",
    "hectoLsun",
    "dasolLum",
    "dekaL_sun",
    "dekaLsun",
    "decaL_sun",
    "decaLsun",
    "dsolLum",
    "deciL_sun",
    "deciLsun",
    "csolLum",
    "centiL_sun",
    "centiLsun",
    "msolLum",
    "milliL_sun",
    "milliLsun",
    "usolLum",
    "μsolLum",
    "microL_sun",
    "microLsun",
    "nsolLum",
    "nanoL_sun",
    "nanoLsun",
    "psolLum",
    "picoL_sun",
    "picoLsun",
    "fsolLum",
    "femtoL_sun",
    "femtoLsun",
    "asolLum",
    "attoL_sun",
    "attoLsun",
    "zsolLum",
    "zeptoL_sun",
    "zeptoLsun",
    "ysolLum",
    "yoctoL_sun",
    "yoctoLsun",
    "rsolLum",
    "rontoL_sun",
    "rontoLsun",
    "qsolLum",
    "quectoL_sun",
    "quectoLsun",
]

from astropy.units.core import PrefixUnit

QsolMass: PrefixUnit
quettaM_sun: PrefixUnit
quettaMsun: PrefixUnit
RsolMass: PrefixUnit
ronnaM_sun: PrefixUnit
ronnaMsun: PrefixUnit
YsolMass: PrefixUnit
yottaM_sun: PrefixUnit
yottaMsun: PrefixUnit
ZsolMass: PrefixUnit
zettaM_sun: PrefixUnit
zettaMsun: PrefixUnit
EsolMass: PrefixUnit
exaM_sun: PrefixUnit
exaMsun: PrefixUnit
PsolMass: PrefixUnit
petaM_sun: PrefixUnit
petaMsun: PrefixUnit
TsolMass: PrefixUnit
teraM_sun: PrefixUnit
teraMsun: PrefixUnit
GsolMass: PrefixUnit
gigaM_sun: PrefixUnit
gigaMsun: PrefixUnit
MsolMass: PrefixUnit
megaM_sun: PrefixUnit
megaMsun: PrefixUnit
ksolMass: PrefixUnit
kiloM_sun: PrefixUnit
kiloMsun: PrefixUnit
hsolMass: PrefixUnit
hectoM_sun: PrefixUnit
hectoMsun: PrefixUnit
dasolMass: PrefixUnit
dekaM_sun: PrefixUnit
dekaMsun: PrefixUnit
decaM_sun: PrefixUnit
decaMsun: PrefixUnit
dsolMass: PrefixUnit
deciM_sun: PrefixUnit
deciMsun: PrefixUnit
csolMass: PrefixUnit
centiM_sun: PrefixUnit
centiMsun: PrefixUnit
msolMass: PrefixUnit
milliM_sun: PrefixUnit
milliMsun: PrefixUnit
usolMass: PrefixUnit
μsolMass: PrefixUnit
microM_sun: PrefixUnit
microMsun: PrefixUnit
nsolMass: PrefixUnit
nanoM_sun: PrefixUnit
nanoMsun: PrefixUnit
psolMass: PrefixUnit
picoM_sun: PrefixUnit
picoMsun: PrefixUnit
fsolMass: PrefixUnit
femtoM_sun: PrefixUnit
femtoMsun: PrefixUnit
asolMass: PrefixUnit
attoM_sun: PrefixUnit
attoMsun: PrefixUnit
zsolMass: PrefixUnit
zeptoM_sun: PrefixUnit
zeptoMsun: PrefixUnit
ysolMass: PrefixUnit
yoctoM_sun: PrefixUnit
yoctoMsun: PrefixUnit
rsolMass: PrefixUnit
rontoM_sun: PrefixUnit
rontoMsun: PrefixUnit
qsolMass: PrefixUnit
quectoM_sun: PrefixUnit
quectoMsun: PrefixUnit
QsolRad: PrefixUnit
quettaR_sun: PrefixUnit
quettaRsun: PrefixUnit
RsolRad: PrefixUnit
ronnaR_sun: PrefixUnit
ronnaRsun: PrefixUnit
YsolRad: PrefixUnit
yottaR_sun: PrefixUnit
yottaRsun: PrefixUnit
ZsolRad: PrefixUnit
zettaR_sun: PrefixUnit
zettaRsun: PrefixUnit
EsolRad: PrefixUnit
exaR_sun: PrefixUnit
exaRsun: PrefixUnit
PsolRad: PrefixUnit
petaR_sun: PrefixUnit
petaRsun: PrefixUnit
TsolRad: PrefixUnit
teraR_sun: PrefixUnit
teraRsun: PrefixUnit
GsolRad: PrefixUnit
gigaR_sun: PrefixUnit
gigaRsun: PrefixUnit
MsolRad: PrefixUnit
megaR_sun: PrefixUnit
megaRsun: PrefixUnit
ksolRad: PrefixUnit
kiloR_sun: PrefixUnit
kiloRsun: PrefixUnit
hsolRad: PrefixUnit
hectoR_sun: PrefixUnit
hectoRsun: PrefixUnit
dasolRad: PrefixUnit
dekaR_sun: PrefixUnit
dekaRsun: PrefixUnit
decaR_sun: PrefixUnit
decaRsun: PrefixUnit
dsolRad: PrefixUnit
deciR_sun: PrefixUnit
deciRsun: PrefixUnit
csolRad: PrefixUnit
centiR_sun: PrefixUnit
centiRsun: PrefixUnit
msolRad: PrefixUnit
milliR_sun: PrefixUnit
milliRsun: PrefixUnit
usolRad: PrefixUnit
μsolRad: PrefixUnit
microR_sun: PrefixUnit
microRsun: PrefixUnit
nsolRad: PrefixUnit
nanoR_sun: PrefixUnit
nanoRsun: PrefixUnit
psolRad: PrefixUnit
picoR_sun: PrefixUnit
picoRsun: PrefixUnit
fsolRad: PrefixUnit
femtoR_sun: PrefixUnit
femtoRsun: PrefixUnit
asolRad: PrefixUnit
attoR_sun: PrefixUnit
attoRsun: PrefixUnit
zsolRad: PrefixUnit
zeptoR_sun: PrefixUnit
zeptoRsun: PrefixUnit
ysolRad: PrefixUnit
yoctoR_sun: PrefixUnit
yoctoRsun: PrefixUnit
rsolRad: PrefixUnit
rontoR_sun: PrefixUnit
rontoRsun: PrefixUnit
qsolRad: PrefixUnit
quectoR_sun: PrefixUnit
quectoRsun: PrefixUnit
QsolLum: PrefixUnit
quettaL_sun: PrefixUnit
quettaLsun: PrefixUnit
RsolLum: PrefixUnit
ronnaL_sun: PrefixUnit
ronnaLsun: PrefixUnit
YsolLum: PrefixUnit
yottaL_sun: PrefixUnit
yottaLsun: PrefixUnit
ZsolLum: PrefixUnit
zettaL_sun: PrefixUnit
zettaLsun: PrefixUnit
EsolLum: PrefixUnit
exaL_sun: PrefixUnit
exaLsun: PrefixUnit
PsolLum: PrefixUnit
petaL_sun: PrefixUnit
petaLsun: PrefixUnit
TsolLum: PrefixUnit
teraL_sun: PrefixUnit
teraLsun: PrefixUnit
GsolLum: PrefixUnit
gigaL_sun: PrefixUnit
gigaLsun: PrefixUnit
MsolLum: PrefixUnit
megaL_sun: PrefixUnit
megaLsun: PrefixUnit
ksolLum: PrefixUnit
kiloL_sun: PrefixUnit
kiloLsun: PrefixUnit
hsolLum: PrefixUnit
hectoL_sun: PrefixUnit
hectoLsun: PrefixUnit
dasolLum: PrefixUnit
dekaL_sun: PrefixUnit
dekaLsun: PrefixUnit
decaL_sun: PrefixUnit
decaLsun: PrefixUnit
dsolLum: PrefixUnit
deciL_sun: PrefixUnit
deciLsun: PrefixUnit
csolLum: PrefixUnit
centiL_sun: PrefixUnit
centiLsun: PrefixUnit
msolLum: PrefixUnit
milliL_sun: PrefixUnit
milliLsun: PrefixUnit
usolLum: PrefixUnit
μsolLum: PrefixUnit
microL_sun: PrefixUnit
microLsun: PrefixUnit
nsolLum: PrefixUnit
nanoL_sun: PrefixUnit
nanoLsun: PrefixUnit
psolLum: PrefixUnit
picoL_sun: PrefixUnit
picoLsun: PrefixUnit
fsolLum: PrefixUnit
femtoL_sun: PrefixUnit
femtoLsun: PrefixUnit
asolLum: PrefixUnit
attoL_sun: PrefixUnit
attoLsun: PrefixUnit
zsolLum: PrefixUnit
zeptoL_sun: PrefixUnit
zeptoLsun: PrefixUnit
ysolLum: PrefixUnit
yoctoL_sun: PrefixUnit
yoctoLsun: PrefixUnit
rsolLum: PrefixUnit
rontoL_sun: PrefixUnit
rontoLsun: PrefixUnit
qsolLum: PrefixUnit
quectoL_sun: PrefixUnit
quectoLsun: PrefixUnit



# This stub file was automatically generated.

__all__ = [
    "cm",
    "g",
    "s",
    "C",
    "rad",
    "sr",
    "cd",
    "K",
    "deg_C",
    "mol",
    "Gal",
    "gal",
    "QGal",
    "quettagal",
    "RGal",
    "ronnagal",
    "YGal",
    "yottagal",
    "ZGal",
    "zettagal",
    "EGal",
    "exagal",
    "PGal",
    "petagal",
    "TGal",
    "teragal",
    "GGal",
    "gigagal",
    "MGal",
    "megagal",
    "kGal",
    "kilogal",
    "hGal",
    "hectogal",
    "daGal",
    "dekagal",
    "decagal",
    "dGal",
    "decigal",
    "cGal",
    "centigal",
    "mGal",
    "milligal",
    "uGal",
    "μGal",
    "microgal",
    "nGal",
    "nanogal",
    "pGal",
    "picogal",
    "fGal",
    "femtogal",
    "aGal",
    "attogal",
    "zGal",
    "zeptogal",
    "yGal",
    "yoctogal",
    "rGal",
    "rontogal",
    "qGal",
    "quectogal",
    "erg",
    "Qerg",
    "Rerg",
    "Yerg",
    "Zerg",
    "Eerg",
    "Perg",
    "Terg",
    "Gerg",
    "Merg",
    "kerg",
    "herg",
    "daerg",
    "derg",
    "cerg",
    "merg",
    "uerg",
    "μerg",
    "nerg",
    "perg",
    "ferg",
    "aerg",
    "zerg",
    "yerg",
    "rerg",
    "qerg",
    "dyn",
    "dyne",
    "Qdyn",
    "quettadyne",
    "Rdyn",
    "ronnadyne",
    "Ydyn",
    "yottadyne",
    "Zdyn",
    "zettadyne",
    "Edyn",
    "exadyne",
    "Pdyn",
    "petadyne",
    "Tdyn",
    "teradyne",
    "Gdyn",
    "gigadyne",
    "Mdyn",
    "megadyne",
    "kdyn",
    "kilodyne",
    "hdyn",
    "hectodyne",
    "dadyn",
    "dekadyne",
    "decadyne",
    "ddyn",
    "decidyne",
    "cdyn",
    "centidyne",
    "mdyn",
    "millidyne",
    "udyn",
    "μdyn",
    "microdyne",
    "ndyn",
    "nanodyne",
    "pdyn",
    "picodyne",
    "fdyn",
    "femtodyne",
    "adyn",
    "attodyne",
    "zdyn",
    "zeptodyne",
    "ydyn",
    "yoctodyne",
    "rdyn",
    "rontodyne",
    "qdyn",
    "quectodyne",
    "Ba",
    "Barye",
    "barye",
    "QBa",
    "quettaBarye",
    "quettabarye",
    "RBa",
    "ronnaBarye",
    "ronnabarye",
    "YBa",
    "yottaBarye",
    "yottabarye",
    "ZBa",
    "zettaBarye",
    "zettabarye",
    "EBa",
    "exaBarye",
    "exabarye",
    "PBa",
    "petaBarye",
    "petabarye",
    "TBa",
    "teraBarye",
    "terabarye",
    "GBa",
    "gigaBarye",
    "gigabarye",
    "MBa",
    "megaBarye",
    "megabarye",
    "kBa",
    "kiloBarye",
    "kilobarye",
    "hBa",
    "hectoBarye",
    "hectobarye",
    "daBa",
    "dekaBarye",
    "dekabarye",
    "decaBarye",
    "decabarye",
    "dBa",
    "deciBarye",
    "decibarye",
    "cBa",
    "centiBarye",
    "centibarye",
    "mBa",
    "milliBarye",
    "millibarye",
    "uBa",
    "μBa",
    "microBarye",
    "microbarye",
    "nBa",
    "nanoBarye",
    "nanobarye",
    "pBa",
    "picoBarye",
    "picobarye",
    "fBa",
    "femtoBarye",
    "femtobarye",
    "aBa",
    "attoBarye",
    "attobarye",
    "zBa",
    "zeptoBarye",
    "zeptobarye",
    "yBa",
    "yoctoBarye",
    "yoctobarye",
    "rBa",
    "rontoBarye",
    "rontobarye",
    "qBa",
    "quectoBarye",
    "quectobarye",
    "P",
    "poise",
    "QP",
    "quettapoise",
    "RP",
    "ronnapoise",
    "YP",
    "yottapoise",
    "ZP",
    "zettapoise",
    "EP",
    "exapoise",
    "PP",
    "petapoise",
    "TP",
    "terapoise",
    "GP",
    "gigapoise",
    "MP",
    "megapoise",
    "kP",
    "kilopoise",
    "hP",
    "hectopoise",
    "daP",
    "dekapoise",
    "decapoise",
    "dP",
    "decipoise",
    "cP",
    "centipoise",
    "mP",
    "millipoise",
    "uP",
    "μP",
    "micropoise",
    "nP",
    "nanopoise",
    "pP",
    "picopoise",
    "fP",
    "femtopoise",
    "aP",
    "attopoise",
    "zP",
    "zeptopoise",
    "yP",
    "yoctopoise",
    "rP",
    "rontopoise",
    "qP",
    "quectopoise",
    "St",
    "stokes",
    "QSt",
    "quettastokes",
    "RSt",
    "ronnastokes",
    "YSt",
    "yottastokes",
    "ZSt",
    "zettastokes",
    "ESt",
    "exastokes",
    "PSt",
    "petastokes",
    "TSt",
    "terastokes",
    "GSt",
    "gigastokes",
    "MSt",
    "megastokes",
    "kSt",
    "kilostokes",
    "hSt",
    "hectostokes",
    "daSt",
    "dekastokes",
    "decastokes",
    "dSt",
    "decistokes",
    "cSt",
    "centistokes",
    "mSt",
    "millistokes",
    "uSt",
    "μSt",
    "microstokes",
    "nSt",
    "nanostokes",
    "pSt",
    "picostokes",
    "fSt",
    "femtostokes",
    "aSt",
    "attostokes",
    "zSt",
    "zeptostokes",
    "ySt",
    "yoctostokes",
    "rSt",
    "rontostokes",
    "qSt",
    "quectostokes",
    "k",
    "Kayser",
    "kayser",
    "Qk",
    "quettaKayser",
    "quettakayser",
    "Rk",
    "ronnaKayser",
    "ronnakayser",
    "Yk",
    "yottaKayser",
    "yottakayser",
    "Zk",
    "zettaKayser",
    "zettakayser",
    "Ek",
    "exaKayser",
    "exakayser",
    "Pk",
    "petaKayser",
    "petakayser",
    "Tk",
    "teraKayser",
    "terakayser",
    "Gk",
    "gigaKayser",
    "gigakayser",
    "Mk",
    "megaKayser",
    "megakayser",
    "kk",
    "kiloKayser",
    "kilokayser",
    "hk",
    "hectoKayser",
    "hectokayser",
    "dak",
    "dekaKayser",
    "dekakayser",
    "decaKayser",
    "decakayser",
    "dk",
    "deciKayser",
    "decikayser",
    "ck",
    "centiKayser",
    "centikayser",
    "mk",
    "milliKayser",
    "millikayser",
    "uk",
    "μk",
    "microKayser",
    "microkayser",
    "nk",
    "nanoKayser",
    "nanokayser",
    "pk",
    "picoKayser",
    "picokayser",
    "fk",
    "femtoKayser",
    "femtokayser",
    "ak",
    "attoKayser",
    "attokayser",
    "zk",
    "zeptoKayser",
    "zeptokayser",
    "yk",
    "yoctoKayser",
    "yoctokayser",
    "rk",
    "rontoKayser",
    "rontokayser",
    "qk",
    "quectoKayser",
    "quectokayser",
    "D",
    "Debye",
    "debye",
    "QD",
    "quettaDebye",
    "quettadebye",
    "RD",
    "ronnaDebye",
    "ronnadebye",
    "YD",
    "yottaDebye",
    "yottadebye",
    "ZD",
    "zettaDebye",
    "zettadebye",
    "ED",
    "exaDebye",
    "exadebye",
    "PD",
    "petaDebye",
    "petadebye",
    "TD",
    "teraDebye",
    "teradebye",
    "GD",
    "gigaDebye",
    "gigadebye",
    "MD",
    "megaDebye",
    "megadebye",
    "kD",
    "kiloDebye",
    "kilodebye",
    "hD",
    "hectoDebye",
    "hectodebye",
    "daD",
    "dekaDebye",
    "dekadebye",
    "decaDebye",
    "decadebye",
    "dD",
    "deciDebye",
    "decidebye",
    "cD",
    "centiDebye",
    "centidebye",
    "mD",
    "milliDebye",
    "millidebye",
    "uD",
    "μD",
    "microDebye",
    "microdebye",
    "nD",
    "nanoDebye",
    "nanodebye",
    "pD",
    "picoDebye",
    "picodebye",
    "fD",
    "femtoDebye",
    "femtodebye",
    "aD",
    "attoDebye",
    "attodebye",
    "zD",
    "zeptoDebye",
    "zeptodebye",
    "yD",
    "yoctoDebye",
    "yoctodebye",
    "rD",
    "rontoDebye",
    "rontodebye",
    "qD",
    "quectoDebye",
    "quectodebye",
    "Fr",
    "Franklin",
    "statcoulomb",
    "statC",
    "esu",
    "statA",
    "statampere",
    "Bi",
    "Biot",
    "abA",
    "abampere",
    "abC",
    "abcoulomb",
    "G",
    "Gauss",
    "gauss",
    "QG",
    "quettaGauss",
    "quettagauss",
    "RG",
    "ronnaGauss",
    "ronnagauss",
    "YG",
    "yottaGauss",
    "yottagauss",
    "ZG",
    "zettaGauss",
    "zettagauss",
    "EG",
    "exaGauss",
    "exagauss",
    "PG",
    "petaGauss",
    "petagauss",
    "TG",
    "teraGauss",
    "teragauss",
    "GG",
    "gigaGauss",
    "gigagauss",
    "MG",
    "megaGauss",
    "megagauss",
    "kG",
    "kiloGauss",
    "kilogauss",
    "hG",
    "hectoGauss",
    "hectogauss",
    "daG",
    "dekaGauss",
    "dekagauss",
    "decaGauss",
    "decagauss",
    "dG",
    "deciGauss",
    "decigauss",
    "cG",
    "centiGauss",
    "centigauss",
    "mG",
    "milliGauss",
    "milligauss",
    "uG",
    "μG",
    "microGauss",
    "microgauss",
    "nG",
    "nanoGauss",
    "nanogauss",
    "pG",
    "picoGauss",
    "picogauss",
    "fG",
    "femtoGauss",
    "femtogauss",
    "aG",
    "attoGauss",
    "attogauss",
    "zG",
    "zeptoGauss",
    "zeptogauss",
    "yG",
    "yoctoGauss",
    "yoctogauss",
    "rG",
    "rontoGauss",
    "rontogauss",
    "qG",
    "quectoGauss",
    "quectogauss",
    "Mx",
    "Maxwell",
    "maxwell",
    "Oe",
    "Oersted",
    "oersted",
    "QOe",
    "quettaOersted",
    "quettaoersted",
    "ROe",
    "ronnaOersted",
    "ronnaoersted",
    "YOe",
    "yottaOersted",
    "yottaoersted",
    "ZOe",
    "zettaOersted",
    "zettaoersted",
    "EOe",
    "exaOersted",
    "exaoersted",
    "POe",
    "petaOersted",
    "petaoersted",
    "TOe",
    "teraOersted",
    "teraoersted",
    "GOe",
    "gigaOersted",
    "gigaoersted",
    "MOe",
    "megaOersted",
    "megaoersted",
    "kOe",
    "kiloOersted",
    "kilooersted",
    "hOe",
    "hectoOersted",
    "hectooersted",
    "daOe",
    "dekaOersted",
    "dekaoersted",
    "decaOersted",
    "decaoersted",
    "dOe",
    "deciOersted",
    "decioersted",
    "cOe",
    "centiOersted",
    "centioersted",
    "mOe",
    "milliOersted",
    "millioersted",
    "uOe",
    "μOe",
    "microOersted",
    "microoersted",
    "nOe",
    "nanoOersted",
    "nanooersted",
    "pOe",
    "picoOersted",
    "picooersted",
    "fOe",
    "femtoOersted",
    "femtooersted",
    "aOe",
    "attoOersted",
    "attooersted",
    "zOe",
    "zeptoOersted",
    "zeptooersted",
    "yOe",
    "yoctoOersted",
    "yoctooersted",
    "rOe",
    "rontoOersted",
    "rontooersted",
    "qOe",
    "quectoOersted",
    "quectooersted",
]

from astropy.units.core import IrreducibleUnit
from astropy.units.core import PrefixUnit
from astropy.units.core import Unit

cm: Unit
g: Unit
s: IrreducibleUnit
C: Unit
rad: IrreducibleUnit
sr: Unit
cd: IrreducibleUnit
K: IrreducibleUnit
deg_C: IrreducibleUnit
mol: IrreducibleUnit
Gal: Unit
gal: Unit
QGal: PrefixUnit
quettagal: PrefixUnit
RGal: PrefixUnit
ronnagal: PrefixUnit
YGal: PrefixUnit
yottagal: PrefixUnit
ZGal: PrefixUnit
zettagal: PrefixUnit
EGal: PrefixUnit
exagal: PrefixUnit
PGal: PrefixUnit
petagal: PrefixUnit
TGal: PrefixUnit
teragal: PrefixUnit
GGal: PrefixUnit
gigagal: PrefixUnit
MGal: PrefixUnit
megagal: PrefixUnit
kGal: PrefixUnit
kilogal: PrefixUnit
hGal: PrefixUnit
hectogal: PrefixUnit
daGal: PrefixUnit
dekagal: PrefixUnit
decagal: PrefixUnit
dGal: PrefixUnit
decigal: PrefixUnit
cGal: PrefixUnit
centigal: PrefixUnit
mGal: PrefixUnit
milligal: PrefixUnit
uGal: PrefixUnit
μGal: PrefixUnit
microgal: PrefixUnit
nGal: PrefixUnit
nanogal: PrefixUnit
pGal: PrefixUnit
picogal: PrefixUnit
fGal: PrefixUnit
femtogal: PrefixUnit
aGal: PrefixUnit
attogal: PrefixUnit
zGal: PrefixUnit
zeptogal: PrefixUnit
yGal: PrefixUnit
yoctogal: PrefixUnit
rGal: PrefixUnit
rontogal: PrefixUnit
qGal: PrefixUnit
quectogal: PrefixUnit
erg: Unit
Qerg: PrefixUnit
Rerg: PrefixUnit
Yerg: PrefixUnit
Zerg: PrefixUnit
Eerg: PrefixUnit
Perg: PrefixUnit
Terg: PrefixUnit
Gerg: PrefixUnit
Merg: PrefixUnit
kerg: PrefixUnit
herg: PrefixUnit
daerg: PrefixUnit
derg: PrefixUnit
cerg: PrefixUnit
merg: PrefixUnit
uerg: PrefixUnit
μerg: PrefixUnit
nerg: PrefixUnit
perg: PrefixUnit
ferg: PrefixUnit
aerg: PrefixUnit
zerg: PrefixUnit
yerg: PrefixUnit
rerg: PrefixUnit
qerg: PrefixUnit
dyn: Unit
dyne: Unit
Qdyn: PrefixUnit
quettadyne: PrefixUnit
Rdyn: PrefixUnit
ronnadyne: PrefixUnit
Ydyn: PrefixUnit
yottadyne: PrefixUnit
Zdyn: PrefixUnit
zettadyne: PrefixUnit
Edyn: PrefixUnit
exadyne: PrefixUnit
Pdyn: PrefixUnit
petadyne: PrefixUnit
Tdyn: PrefixUnit
teradyne: PrefixUnit
Gdyn: PrefixUnit
gigadyne: PrefixUnit
Mdyn: PrefixUnit
megadyne: PrefixUnit
kdyn: PrefixUnit
kilodyne: PrefixUnit
hdyn: PrefixUnit
hectodyne: PrefixUnit
dadyn: PrefixUnit
dekadyne: PrefixUnit
decadyne: PrefixUnit
ddyn: PrefixUnit
decidyne: PrefixUnit
cdyn: PrefixUnit
centidyne: PrefixUnit
mdyn: PrefixUnit
millidyne: PrefixUnit
udyn: PrefixUnit
μdyn: PrefixUnit
microdyne: PrefixUnit
ndyn: PrefixUnit
nanodyne: PrefixUnit
pdyn: PrefixUnit
picodyne: PrefixUnit
fdyn: PrefixUnit
femtodyne: PrefixUnit
adyn: PrefixUnit
attodyne: PrefixUnit
zdyn: PrefixUnit
zeptodyne: PrefixUnit
ydyn: PrefixUnit
yoctodyne: PrefixUnit
rdyn: PrefixUnit
rontodyne: PrefixUnit
qdyn: PrefixUnit
quectodyne: PrefixUnit
Ba: Unit
Barye: Unit
barye: Unit
QBa: PrefixUnit
quettaBarye: PrefixUnit
quettabarye: PrefixUnit
RBa: PrefixUnit
ronnaBarye: PrefixUnit
ronnabarye: PrefixUnit
YBa: PrefixUnit
yottaBarye: PrefixUnit
yottabarye: PrefixUnit
ZBa: PrefixUnit
zettaBarye: PrefixUnit
zettabarye: PrefixUnit
EBa: PrefixUnit
exaBarye: PrefixUnit
exabarye: PrefixUnit
PBa: PrefixUnit
petaBarye: PrefixUnit
petabarye: PrefixUnit
TBa: PrefixUnit
teraBarye: PrefixUnit
terabarye: PrefixUnit
GBa: PrefixUnit
gigaBarye: PrefixUnit
gigabarye: PrefixUnit
MBa: PrefixUnit
megaBarye: PrefixUnit
megabarye: PrefixUnit
kBa: PrefixUnit
kiloBarye: PrefixUnit
kilobarye: PrefixUnit
hBa: PrefixUnit
hectoBarye: PrefixUnit
hectobarye: PrefixUnit
daBa: PrefixUnit
dekaBarye: PrefixUnit
dekabarye: PrefixUnit
decaBarye: PrefixUnit
decabarye: PrefixUnit
dBa: PrefixUnit
deciBarye: PrefixUnit
decibarye: PrefixUnit
cBa: PrefixUnit
centiBarye: PrefixUnit
centibarye: PrefixUnit
mBa: PrefixUnit
milliBarye: PrefixUnit
millibarye: PrefixUnit
uBa: PrefixUnit
μBa: PrefixUnit
microBarye: PrefixUnit
microbarye: PrefixUnit
nBa: PrefixUnit
nanoBarye: PrefixUnit
nanobarye: PrefixUnit
pBa: PrefixUnit
picoBarye: PrefixUnit
picobarye: PrefixUnit
fBa: PrefixUnit
femtoBarye: PrefixUnit
femtobarye: PrefixUnit
aBa: PrefixUnit
attoBarye: PrefixUnit
attobarye: PrefixUnit
zBa: PrefixUnit
zeptoBarye: PrefixUnit
zeptobarye: PrefixUnit
yBa: PrefixUnit
yoctoBarye: PrefixUnit
yoctobarye: PrefixUnit
rBa: PrefixUnit
rontoBarye: PrefixUnit
rontobarye: PrefixUnit
qBa: PrefixUnit
quectoBarye: PrefixUnit
quectobarye: PrefixUnit
P: Unit
poise: Unit
QP: PrefixUnit
quettapoise: PrefixUnit
RP: PrefixUnit
ronnapoise: PrefixUnit
YP: PrefixUnit
yottapoise: PrefixUnit
ZP: PrefixUnit
zettapoise: PrefixUnit
EP: PrefixUnit
exapoise: PrefixUnit
PP: PrefixUnit
petapoise: PrefixUnit
TP: PrefixUnit
terapoise: PrefixUnit
GP: PrefixUnit
gigapoise: PrefixUnit
MP: PrefixUnit
megapoise: PrefixUnit
kP: PrefixUnit
kilopoise: PrefixUnit
hP: PrefixUnit
hectopoise: PrefixUnit
daP: PrefixUnit
dekapoise: PrefixUnit
decapoise: PrefixUnit
dP: PrefixUnit
decipoise: PrefixUnit
cP: PrefixUnit
centipoise: PrefixUnit
mP: PrefixUnit
millipoise: PrefixUnit
uP: PrefixUnit
μP: PrefixUnit
micropoise: PrefixUnit
nP: PrefixUnit
nanopoise: PrefixUnit
pP: PrefixUnit
picopoise: PrefixUnit
fP: PrefixUnit
femtopoise: PrefixUnit
aP: PrefixUnit
attopoise: PrefixUnit
zP: PrefixUnit
zeptopoise: PrefixUnit
yP: PrefixUnit
yoctopoise: PrefixUnit
rP: PrefixUnit
rontopoise: PrefixUnit
qP: PrefixUnit
quectopoise: PrefixUnit
St: Unit
stokes: Unit
QSt: PrefixUnit
quettastokes: PrefixUnit
RSt: PrefixUnit
ronnastokes: PrefixUnit
YSt: PrefixUnit
yottastokes: PrefixUnit
ZSt: PrefixUnit
zettastokes: PrefixUnit
ESt: PrefixUnit
exastokes: PrefixUnit
PSt: PrefixUnit
petastokes: PrefixUnit
TSt: PrefixUnit
terastokes: PrefixUnit
GSt: PrefixUnit
gigastokes: PrefixUnit
MSt: PrefixUnit
megastokes: PrefixUnit
kSt: PrefixUnit
kilostokes: PrefixUnit
hSt: PrefixUnit
hectostokes: PrefixUnit
daSt: PrefixUnit
dekastokes: PrefixUnit
decastokes: PrefixUnit
dSt: PrefixUnit
decistokes: PrefixUnit
cSt: PrefixUnit
centistokes: PrefixUnit
mSt: PrefixUnit
millistokes: PrefixUnit
uSt: PrefixUnit
μSt: PrefixUnit
microstokes: PrefixUnit
nSt: PrefixUnit
nanostokes: PrefixUnit
pSt: PrefixUnit
picostokes: PrefixUnit
fSt: PrefixUnit
femtostokes: PrefixUnit
aSt: PrefixUnit
attostokes: PrefixUnit
zSt: PrefixUnit
zeptostokes: PrefixUnit
ySt: PrefixUnit
yoctostokes: PrefixUnit
rSt: PrefixUnit
rontostokes: PrefixUnit
qSt: PrefixUnit
quectostokes: PrefixUnit
k: Unit
Kayser: Unit
kayser: Unit
Qk: PrefixUnit
quettaKayser: PrefixUnit
quettakayser: PrefixUnit
Rk: PrefixUnit
ronnaKayser: PrefixUnit
ronnakayser: PrefixUnit
Yk: PrefixUnit
yottaKayser: PrefixUnit
yottakayser: PrefixUnit
Zk: PrefixUnit
zettaKayser: PrefixUnit
zettakayser: PrefixUnit
Ek: PrefixUnit
exaKayser: PrefixUnit
exakayser: PrefixUnit
Pk: PrefixUnit
petaKayser: PrefixUnit
petakayser: PrefixUnit
Tk: PrefixUnit
teraKayser: PrefixUnit
terakayser: PrefixUnit
Gk: PrefixUnit
gigaKayser: PrefixUnit
gigakayser: PrefixUnit
Mk: PrefixUnit
megaKayser: PrefixUnit
megakayser: PrefixUnit
kk: PrefixUnit
kiloKayser: PrefixUnit
kilokayser: PrefixUnit
hk: PrefixUnit
hectoKayser: PrefixUnit
hectokayser: PrefixUnit
dak: PrefixUnit
dekaKayser: PrefixUnit
dekakayser: PrefixUnit
decaKayser: PrefixUnit
decakayser: PrefixUnit
dk: PrefixUnit
deciKayser: PrefixUnit
decikayser: PrefixUnit
ck: PrefixUnit
centiKayser: PrefixUnit
centikayser: PrefixUnit
mk: PrefixUnit
milliKayser: PrefixUnit
millikayser: PrefixUnit
uk: PrefixUnit
μk: PrefixUnit
microKayser: PrefixUnit
microkayser: PrefixUnit
nk: PrefixUnit
nanoKayser: PrefixUnit
nanokayser: PrefixUnit
pk: PrefixUnit
picoKayser: PrefixUnit
picokayser: PrefixUnit
fk: PrefixUnit
femtoKayser: PrefixUnit
femtokayser: PrefixUnit
ak: PrefixUnit
attoKayser: PrefixUnit
attokayser: PrefixUnit
zk: PrefixUnit
zeptoKayser: PrefixUnit
zeptokayser: PrefixUnit
yk: PrefixUnit
yoctoKayser: PrefixUnit
yoctokayser: PrefixUnit
rk: PrefixUnit
rontoKayser: PrefixUnit
rontokayser: PrefixUnit
qk: PrefixUnit
quectoKayser: PrefixUnit
quectokayser: PrefixUnit
D: Unit
Debye: Unit
debye: Unit
QD: PrefixUnit
quettaDebye: PrefixUnit
quettadebye: PrefixUnit
RD: PrefixUnit
ronnaDebye: PrefixUnit
ronnadebye: PrefixUnit
YD: PrefixUnit
yottaDebye: PrefixUnit
yottadebye: PrefixUnit
ZD: PrefixUnit
zettaDebye: PrefixUnit
zettadebye: PrefixUnit
ED: PrefixUnit
exaDebye: PrefixUnit
exadebye: PrefixUnit
PD: PrefixUnit
petaDebye: PrefixUnit
petadebye: PrefixUnit
TD: PrefixUnit
teraDebye: PrefixUnit
teradebye: PrefixUnit
GD: PrefixUnit
gigaDebye: PrefixUnit
gigadebye: PrefixUnit
MD: PrefixUnit
megaDebye: PrefixUnit
megadebye: PrefixUnit
kD: PrefixUnit
kiloDebye: PrefixUnit
kilodebye: PrefixUnit
hD: PrefixUnit
hectoDebye: PrefixUnit
hectodebye: PrefixUnit
daD: PrefixUnit
dekaDebye: PrefixUnit
dekadebye: PrefixUnit
decaDebye: PrefixUnit
decadebye: PrefixUnit
dD: PrefixUnit
deciDebye: PrefixUnit
decidebye: PrefixUnit
cD: PrefixUnit
centiDebye: PrefixUnit
centidebye: PrefixUnit
mD: PrefixUnit
milliDebye: PrefixUnit
millidebye: PrefixUnit
uD: PrefixUnit
μD: PrefixUnit
microDebye: PrefixUnit
microdebye: PrefixUnit
nD: PrefixUnit
nanoDebye: PrefixUnit
nanodebye: PrefixUnit
pD: PrefixUnit
picoDebye: PrefixUnit
picodebye: PrefixUnit
fD: PrefixUnit
femtoDebye: PrefixUnit
femtodebye: PrefixUnit
aD: PrefixUnit
attoDebye: PrefixUnit
attodebye: PrefixUnit
zD: PrefixUnit
zeptoDebye: PrefixUnit
zeptodebye: PrefixUnit
yD: PrefixUnit
yoctoDebye: PrefixUnit
yoctodebye: PrefixUnit
rD: PrefixUnit
rontoDebye: PrefixUnit
rontodebye: PrefixUnit
qD: PrefixUnit
quectoDebye: PrefixUnit
quectodebye: PrefixUnit
Fr: Unit
Franklin: Unit
statcoulomb: Unit
statC: Unit
esu: Unit
statA: Unit
statampere: Unit
Bi: Unit
Biot: Unit
abA: Unit
abampere: Unit
abC: Unit
abcoulomb: Unit
G: Unit
Gauss: Unit
gauss: Unit
QG: PrefixUnit
quettaGauss: PrefixUnit
quettagauss: PrefixUnit
RG: PrefixUnit
ronnaGauss: PrefixUnit
ronnagauss: PrefixUnit
YG: PrefixUnit
yottaGauss: PrefixUnit
yottagauss: PrefixUnit
ZG: PrefixUnit
zettaGauss: PrefixUnit
zettagauss: PrefixUnit
EG: PrefixUnit
exaGauss: PrefixUnit
exagauss: PrefixUnit
PG: PrefixUnit
petaGauss: PrefixUnit
petagauss: PrefixUnit
TG: PrefixUnit
teraGauss: PrefixUnit
teragauss: PrefixUnit
GG: PrefixUnit
gigaGauss: PrefixUnit
gigagauss: PrefixUnit
MG: PrefixUnit
megaGauss: PrefixUnit
megagauss: PrefixUnit
kG: PrefixUnit
kiloGauss: PrefixUnit
kilogauss: PrefixUnit
hG: PrefixUnit
hectoGauss: PrefixUnit
hectogauss: PrefixUnit
daG: PrefixUnit
dekaGauss: PrefixUnit
dekagauss: PrefixUnit
decaGauss: PrefixUnit
decagauss: PrefixUnit
dG: PrefixUnit
deciGauss: PrefixUnit
decigauss: PrefixUnit
cG: PrefixUnit
centiGauss: PrefixUnit
centigauss: PrefixUnit
mG: PrefixUnit
milliGauss: PrefixUnit
milligauss: PrefixUnit
uG: PrefixUnit
μG: PrefixUnit
microGauss: PrefixUnit
microgauss: PrefixUnit
nG: PrefixUnit
nanoGauss: PrefixUnit
nanogauss: PrefixUnit
pG: PrefixUnit
picoGauss: PrefixUnit
picogauss: PrefixUnit
fG: PrefixUnit
femtoGauss: PrefixUnit
femtogauss: PrefixUnit
aG: PrefixUnit
attoGauss: PrefixUnit
attogauss: PrefixUnit
zG: PrefixUnit
zeptoGauss: PrefixUnit
zeptogauss: PrefixUnit
yG: PrefixUnit
yoctoGauss: PrefixUnit
yoctogauss: PrefixUnit
rG: PrefixUnit
rontoGauss: PrefixUnit
rontogauss: PrefixUnit
qG: PrefixUnit
quectoGauss: PrefixUnit
quectogauss: PrefixUnit
Mx: Unit
Maxwell: Unit
maxwell: Unit
Oe: Unit
Oersted: Unit
oersted: Unit
QOe: PrefixUnit
quettaOersted: PrefixUnit
quettaoersted: PrefixUnit
ROe: PrefixUnit
ronnaOersted: PrefixUnit
ronnaoersted: PrefixUnit
YOe: PrefixUnit
yottaOersted: PrefixUnit
yottaoersted: PrefixUnit
ZOe: PrefixUnit
zettaOersted: PrefixUnit
zettaoersted: PrefixUnit
EOe: PrefixUnit
exaOersted: PrefixUnit
exaoersted: PrefixUnit
POe: PrefixUnit
petaOersted: PrefixUnit
petaoersted: PrefixUnit
TOe: PrefixUnit
teraOersted: PrefixUnit
teraoersted: PrefixUnit
GOe: PrefixUnit
gigaOersted: PrefixUnit
gigaoersted: PrefixUnit
MOe: PrefixUnit
megaOersted: PrefixUnit
megaoersted: PrefixUnit
kOe: PrefixUnit
kiloOersted: PrefixUnit
kilooersted: PrefixUnit
hOe: PrefixUnit
hectoOersted: PrefixUnit
hectooersted: PrefixUnit
daOe: PrefixUnit
dekaOersted: PrefixUnit
dekaoersted: PrefixUnit
decaOersted: PrefixUnit
decaoersted: PrefixUnit
dOe: PrefixUnit
deciOersted: PrefixUnit
decioersted: PrefixUnit
cOe: PrefixUnit
centiOersted: PrefixUnit
centioersted: PrefixUnit
mOe: PrefixUnit
milliOersted: PrefixUnit
millioersted: PrefixUnit
uOe: PrefixUnit
μOe: PrefixUnit
microOersted: PrefixUnit
microoersted: PrefixUnit
nOe: PrefixUnit
nanoOersted: PrefixUnit
nanooersted: PrefixUnit
pOe: PrefixUnit
picoOersted: PrefixUnit
picooersted: PrefixUnit
fOe: PrefixUnit
femtoOersted: PrefixUnit
femtooersted: PrefixUnit
aOe: PrefixUnit
attoOersted: PrefixUnit
attooersted: PrefixUnit
zOe: PrefixUnit
zeptoOersted: PrefixUnit
zeptooersted: PrefixUnit
yOe: PrefixUnit
yoctoOersted: PrefixUnit
yoctooersted: PrefixUnit
rOe: PrefixUnit
rontoOersted: PrefixUnit
rontooersted: PrefixUnit
qOe: PrefixUnit
quectoOersted: PrefixUnit
quectooersted: PrefixUnit



# This stub file was automatically generated.

__all__ = [
    "enable",
    "inch",
    "ft",
    "foot",
    "yd",
    "yard",
    "mi",
    "mile",
    "mil",
    "thou",
    "nmi",
    "nauticalmile",
    "NM",
    "fur",
    "furlong",
    "ac",
    "acre",
    "gallon",
    "quart",
    "pint",
    "cup",
    "foz",
    "fluid_oz",
    "fluid_ounce",
    "tbsp",
    "tablespoon",
    "tsp",
    "teaspoon",
    "oz",
    "ounce",
    "lb",
    "lbm",
    "pound",
    "st",
    "stone",
    "ton",
    "slug",
    "kn",
    "kt",
    "knot",
    "NMPH",
    "lbf",
    "kip",
    "kilopound",
    "BTU",
    "btu",
    "cal",
    "calorie",
    "kcal",
    "Cal",
    "Calorie",
    "kilocal",
    "kilocalorie",
    "psi",
    "hp",
    "horsepower",
    "deg_F",
    "Fahrenheit",
    "deg_R",
    "Rankine",
]

from astropy.units.core import IrreducibleUnit
from astropy.units.core import Unit
from astropy.units.core import _UnitContext

inch: Unit
ft: Unit
foot: Unit
yd: Unit
yard: Unit
mi: Unit
mile: Unit
mil: Unit
thou: Unit
nmi: Unit
nauticalmile: Unit
NM: Unit
fur: Unit
furlong: Unit
ac: Unit
acre: Unit
gallon: Unit
quart: Unit
pint: Unit
cup: Unit
foz: Unit
fluid_oz: Unit
fluid_ounce: Unit
tbsp: Unit
tablespoon: Unit
tsp: Unit
teaspoon: Unit
oz: Unit
ounce: Unit
lb: Unit
lbm: Unit
pound: Unit
st: Unit
stone: Unit
ton: Unit
slug: Unit
kn: Unit
kt: Unit
knot: Unit
NMPH: Unit
lbf: Unit
kip: Unit
kilopound: Unit
BTU: Unit
btu: Unit
cal: Unit
calorie: Unit
kcal: Unit
Cal: Unit
Calorie: Unit
kilocal: Unit
kilocalorie: Unit
psi: Unit
hp: Unit
horsepower: Unit
deg_F: IrreducibleUnit
Fahrenheit: IrreducibleUnit
deg_R: Unit
Rankine: Unit

def enable() -> _UnitContext: ...



# This stub file was automatically generated.

__all__ = [
    "AU",
    "au",
    "astronomical_unit",
    "QAU",
    "Qau",
    "quettaastronomical_unit",
    "RAU",
    "Rau",
    "ronnaastronomical_unit",
    "YAU",
    "Yau",
    "yottaastronomical_unit",
    "ZAU",
    "Zau",
    "zettaastronomical_unit",
    "EAU",
    "Eau",
    "exaastronomical_unit",
    "PAU",
    "Pau",
    "petaastronomical_unit",
    "TAU",
    "Tau",
    "teraastronomical_unit",
    "GAU",
    "Gau",
    "gigaastronomical_unit",
    "MAU",
    "Mau",
    "megaastronomical_unit",
    "kAU",
    "kau",
    "kiloastronomical_unit",
    "hAU",
    "hau",
    "hectoastronomical_unit",
    "daAU",
    "daau",
    "dekaastronomical_unit",
    "decaastronomical_unit",
    "dAU",
    "dau",
    "deciastronomical_unit",
    "cAU",
    "cau",
    "centiastronomical_unit",
    "mAU",
    "mau",
    "milliastronomical_unit",
    "uAU",
    "uau",
    "μAU",
    "μau",
    "microastronomical_unit",
    "nAU",
    "nau",
    "nanoastronomical_unit",
    "pAU",
    "pau",
    "picoastronomical_unit",
    "fAU",
    "fau",
    "femtoastronomical_unit",
    "aAU",
    "aau",
    "attoastronomical_unit",
    "zAU",
    "zau",
    "zeptoastronomical_unit",
    "yAU",
    "yau",
    "yoctoastronomical_unit",
    "rAU",
    "rau",
    "rontoastronomical_unit",
    "qAU",
    "qau",
    "quectoastronomical_unit",
    "pc",
    "parsec",
    "Qpc",
    "quettaparsec",
    "Rpc",
    "ronnaparsec",
    "Ypc",
    "yottaparsec",
    "Zpc",
    "zettaparsec",
    "Epc",
    "exaparsec",
    "Ppc",
    "petaparsec",
    "Tpc",
    "teraparsec",
    "Gpc",
    "gigaparsec",
    "Mpc",
    "megaparsec",
    "kpc",
    "kiloparsec",
    "hpc",
    "hectoparsec",
    "dapc",
    "dekaparsec",
    "decaparsec",
    "dpc",
    "deciparsec",
    "cpc",
    "centiparsec",
    "mpc",
    "milliparsec",
    "upc",
    "μpc",
    "microparsec",
    "npc",
    "nanoparsec",
    "ppc",
    "picoparsec",
    "fpc",
    "femtoparsec",
    "apc",
    "attoparsec",
    "zpc",
    "zeptoparsec",
    "ypc",
    "yoctoparsec",
    "rpc",
    "rontoparsec",
    "qpc",
    "quectoparsec",
    "solRad",
    "R_sun",
    "Rsun",
    "jupiterRad",
    "R_jup",
    "Rjup",
    "R_jupiter",
    "Rjupiter",
    "earthRad",
    "R_earth",
    "Rearth",
    "lyr",
    "lightyear",
    "Qlyr",
    "quettalightyear",
    "Rlyr",
    "ronnalightyear",
    "Ylyr",
    "yottalightyear",
    "Zlyr",
    "zettalightyear",
    "Elyr",
    "exalightyear",
    "Plyr",
    "petalightyear",
    "Tlyr",
    "teralightyear",
    "Glyr",
    "gigalightyear",
    "Mlyr",
    "megalightyear",
    "klyr",
    "kilolightyear",
    "hlyr",
    "hectolightyear",
    "dalyr",
    "dekalightyear",
    "decalightyear",
    "dlyr",
    "decilightyear",
    "clyr",
    "centilightyear",
    "mlyr",
    "millilightyear",
    "ulyr",
    "μlyr",
    "microlightyear",
    "nlyr",
    "nanolightyear",
    "plyr",
    "picolightyear",
    "flyr",
    "femtolightyear",
    "alyr",
    "attolightyear",
    "zlyr",
    "zeptolightyear",
    "ylyr",
    "yoctolightyear",
    "rlyr",
    "rontolightyear",
    "qlyr",
    "quectolightyear",
    "lsec",
    "lightsecond",
    "solMass",
    "M_sun",
    "Msun",
    "jupiterMass",
    "M_jup",
    "Mjup",
    "M_jupiter",
    "Mjupiter",
    "earthMass",
    "M_earth",
    "Mearth",
    "foe",
    "Bethe",
    "bethe",
    "solLum",
    "L_sun",
    "Lsun",
    "ph",
    "photon",
    "Qph",
    "Qphoton",
    "quettaphoton",
    "Rph",
    "Rphoton",
    "ronnaphoton",
    "Yph",
    "Yphoton",
    "yottaphoton",
    "Zph",
    "Zphoton",
    "zettaphoton",
    "Eph",
    "Ephoton",
    "exaphoton",
    "Pph",
    "Pphoton",
    "petaphoton",
    "Tph",
    "Tphoton",
    "teraphoton",
    "Gph",
    "Gphoton",
    "gigaphoton",
    "Mph",
    "Mphoton",
    "megaphoton",
    "kph",
    "kphoton",
    "kilophoton",
    "hph",
    "hphoton",
    "hectophoton",
    "daph",
    "daphoton",
    "dekaphoton",
    "decaphoton",
    "dph",
    "dphoton",
    "deciphoton",
    "cph",
    "cphoton",
    "centiphoton",
    "mph",
    "mphoton",
    "milliphoton",
    "uph",
    "uphoton",
    "μph",
    "μphoton",
    "microphoton",
    "nph",
    "nphoton",
    "nanophoton",
    "pph",
    "pphoton",
    "picophoton",
    "fph",
    "fphoton",
    "femtophoton",
    "aph",
    "aphoton",
    "attophoton",
    "zph",
    "zphoton",
    "zeptophoton",
    "yph",
    "yphoton",
    "yoctophoton",
    "rph",
    "rphoton",
    "rontophoton",
    "qph",
    "qphoton",
    "quectophoton",
    "Jy",
    "Jansky",
    "jansky",
    "QJy",
    "quettaJansky",
    "quettajansky",
    "RJy",
    "ronnaJansky",
    "ronnajansky",
    "YJy",
    "yottaJansky",
    "yottajansky",
    "ZJy",
    "zettaJansky",
    "zettajansky",
    "EJy",
    "exaJansky",
    "exajansky",
    "PJy",
    "petaJansky",
    "petajansky",
    "TJy",
    "teraJansky",
    "terajansky",
    "GJy",
    "gigaJansky",
    "gigajansky",
    "MJy",
    "megaJansky",
    "megajansky",
    "kJy",
    "kiloJansky",
    "kilojansky",
    "hJy",
    "hectoJansky",
    "hectojansky",
    "daJy",
    "dekaJansky",
    "dekajansky",
    "decaJansky",
    "decajansky",
    "dJy",
    "deciJansky",
    "decijansky",
    "cJy",
    "centiJansky",
    "centijansky",
    "mJy",
    "milliJansky",
    "millijansky",
    "uJy",
    "μJy",
    "microJansky",
    "microjansky",
    "nJy",
    "nanoJansky",
    "nanojansky",
    "pJy",
    "picoJansky",
    "picojansky",
    "fJy",
    "femtoJansky",
    "femtojansky",
    "aJy",
    "attoJansky",
    "attojansky",
    "zJy",
    "zeptoJansky",
    "zeptojansky",
    "yJy",
    "yoctoJansky",
    "yoctojansky",
    "rJy",
    "rontoJansky",
    "rontojansky",
    "qJy",
    "quectoJansky",
    "quectojansky",
    "R",
    "Rayleigh",
    "rayleigh",
    "QR",
    "quettaRayleigh",
    "quettarayleigh",
    "RR",
    "ronnaRayleigh",
    "ronnarayleigh",
    "YR",
    "yottaRayleigh",
    "yottarayleigh",
    "ZR",
    "zettaRayleigh",
    "zettarayleigh",
    "ER",
    "exaRayleigh",
    "exarayleigh",
    "PR",
    "petaRayleigh",
    "petarayleigh",
    "TR",
    "teraRayleigh",
    "terarayleigh",
    "GR",
    "gigaRayleigh",
    "gigarayleigh",
    "MR",
    "megaRayleigh",
    "megarayleigh",
    "kR",
    "kiloRayleigh",
    "kilorayleigh",
    "hR",
    "hectoRayleigh",
    "hectorayleigh",
    "daR",
    "dekaRayleigh",
    "dekarayleigh",
    "decaRayleigh",
    "decarayleigh",
    "dR",
    "deciRayleigh",
    "decirayleigh",
    "cR",
    "centiRayleigh",
    "centirayleigh",
    "mR",
    "milliRayleigh",
    "millirayleigh",
    "uR",
    "μR",
    "microRayleigh",
    "microrayleigh",
    "nR",
    "nanoRayleigh",
    "nanorayleigh",
    "pR",
    "picoRayleigh",
    "picorayleigh",
    "fR",
    "femtoRayleigh",
    "femtorayleigh",
    "aR",
    "attoRayleigh",
    "attorayleigh",
    "zR",
    "zeptoRayleigh",
    "zeptorayleigh",
    "yR",
    "yoctoRayleigh",
    "yoctorayleigh",
    "rR",
    "rontoRayleigh",
    "rontorayleigh",
    "qR",
    "quectoRayleigh",
    "quectorayleigh",
    "ct",
    "count",
    "Qct",
    "Qcount",
    "quettacount",
    "Rct",
    "Rcount",
    "ronnacount",
    "Yct",
    "Ycount",
    "yottacount",
    "Zct",
    "Zcount",
    "zettacount",
    "Ect",
    "Ecount",
    "exacount",
    "Pct",
    "Pcount",
    "petacount",
    "Tct",
    "Tcount",
    "teracount",
    "Gct",
    "Gcount",
    "gigacount",
    "Mct",
    "Mcount",
    "megacount",
    "kct",
    "kcount",
    "kilocount",
    "hct",
    "hcount",
    "hectocount",
    "dact",
    "dacount",
    "dekacount",
    "decacount",
    "dct",
    "dcount",
    "decicount",
    "cct",
    "ccount",
    "centicount",
    "mct",
    "mcount",
    "millicount",
    "uct",
    "ucount",
    "μct",
    "μcount",
    "microcount",
    "nct",
    "ncount",
    "nanocount",
    "picocount",
    "fct",
    "fcount",
    "femtocount",
    "act",
    "acount",
    "attocount",
    "zct",
    "zcount",
    "zeptocount",
    "yct",
    "ycount",
    "yoctocount",
    "rct",
    "rcount",
    "rontocount",
    "qct",
    "qcount",
    "quectocount",
    "adu",
    "Qadu",
    "Radu",
    "Yadu",
    "Zadu",
    "Eadu",
    "Padu",
    "Tadu",
    "Gadu",
    "Madu",
    "kadu",
    "hadu",
    "daadu",
    "dadu",
    "cadu",
    "madu",
    "uadu",
    "μadu",
    "nadu",
    "padu",
    "fadu",
    "aadu",
    "zadu",
    "yadu",
    "radu",
    "qadu",
    "DN",
    "dn",
    "Sun",
    "chan",
    "Qchan",
    "Rchan",
    "Ychan",
    "Zchan",
    "Echan",
    "Pchan",
    "Tchan",
    "Gchan",
    "Mchan",
    "kchan",
    "hchan",
    "dachan",
    "dchan",
    "cchan",
    "mchan",
    "uchan",
    "μchan",
    "nchan",
    "pchan",
    "fchan",
    "achan",
    "zchan",
    "ychan",
    "rchan",
    "qchan",
    "bin",
    "Qbin",
    "Rbin",
    "Ybin",
    "Zbin",
    "Ebin",
    "Pbin",
    "Tbin",
    "Gbin",
    "Mbin",
    "kbin",
    "hbin",
    "dabin",
    "dbin",
    "cbin",
    "mbin",
    "ubin",
    "μbin",
    "nbin",
    "pbin",
    "fbin",
    "abin",
    "zbin",
    "ybin",
    "rbin",
    "qbin",
    "beam",
    "Qbeam",
    "Rbeam",
    "Ybeam",
    "Zbeam",
    "Ebeam",
    "Pbeam",
    "Tbeam",
    "Gbeam",
    "Mbeam",
    "kbeam",
    "hbeam",
    "dabeam",
    "dbeam",
    "cbeam",
    "mbeam",
    "ubeam",
    "μbeam",
    "nbeam",
    "pbeam",
    "fbeam",
    "abeam",
    "zbeam",
    "ybeam",
    "rbeam",
    "qbeam",
    "electron",
]

from astropy.units.core import IrreducibleUnit
from astropy.units.core import PrefixUnit
from astropy.units.core import Unit

AU: Unit
au: Unit
astronomical_unit: Unit
QAU: PrefixUnit
Qau: PrefixUnit
quettaastronomical_unit: PrefixUnit
RAU: PrefixUnit
Rau: PrefixUnit
ronnaastronomical_unit: PrefixUnit
YAU: PrefixUnit
Yau: PrefixUnit
yottaastronomical_unit: PrefixUnit
ZAU: PrefixUnit
Zau: PrefixUnit
zettaastronomical_unit: PrefixUnit
EAU: PrefixUnit
Eau: PrefixUnit
exaastronomical_unit: PrefixUnit
PAU: PrefixUnit
Pau: PrefixUnit
petaastronomical_unit: PrefixUnit
TAU: PrefixUnit
Tau: PrefixUnit
teraastronomical_unit: PrefixUnit
GAU: PrefixUnit
Gau: PrefixUnit
gigaastronomical_unit: PrefixUnit
MAU: PrefixUnit
Mau: PrefixUnit
megaastronomical_unit: PrefixUnit
kAU: PrefixUnit
kau: PrefixUnit
kiloastronomical_unit: PrefixUnit
hAU: PrefixUnit
hau: PrefixUnit
hectoastronomical_unit: PrefixUnit
daAU: PrefixUnit
daau: PrefixUnit
dekaastronomical_unit: PrefixUnit
decaastronomical_unit: PrefixUnit
dAU: PrefixUnit
dau: PrefixUnit
deciastronomical_unit: PrefixUnit
cAU: PrefixUnit
cau: PrefixUnit
centiastronomical_unit: PrefixUnit
mAU: PrefixUnit
mau: PrefixUnit
milliastronomical_unit: PrefixUnit
uAU: PrefixUnit
uau: PrefixUnit
μAU: PrefixUnit
μau: PrefixUnit
microastronomical_unit: PrefixUnit
nAU: PrefixUnit
nau: PrefixUnit
nanoastronomical_unit: PrefixUnit
pAU: PrefixUnit
pau: PrefixUnit
picoastronomical_unit: PrefixUnit
fAU: PrefixUnit
fau: PrefixUnit
femtoastronomical_unit: PrefixUnit
aAU: PrefixUnit
aau: PrefixUnit
attoastronomical_unit: PrefixUnit
zAU: PrefixUnit
zau: PrefixUnit
zeptoastronomical_unit: PrefixUnit
yAU: PrefixUnit
yau: PrefixUnit
yoctoastronomical_unit: PrefixUnit
rAU: PrefixUnit
rau: PrefixUnit
rontoastronomical_unit: PrefixUnit
qAU: PrefixUnit
qau: PrefixUnit
quectoastronomical_unit: PrefixUnit
pc: Unit
parsec: Unit
Qpc: PrefixUnit
quettaparsec: PrefixUnit
Rpc: PrefixUnit
ronnaparsec: PrefixUnit
Ypc: PrefixUnit
yottaparsec: PrefixUnit
Zpc: PrefixUnit
zettaparsec: PrefixUnit
Epc: PrefixUnit
exaparsec: PrefixUnit
Ppc: PrefixUnit
petaparsec: PrefixUnit
Tpc: PrefixUnit
teraparsec: PrefixUnit
Gpc: PrefixUnit
gigaparsec: PrefixUnit
Mpc: PrefixUnit
megaparsec: PrefixUnit
kpc: PrefixUnit
kiloparsec: PrefixUnit
hpc: PrefixUnit
hectoparsec: PrefixUnit
dapc: PrefixUnit
dekaparsec: PrefixUnit
decaparsec: PrefixUnit
dpc: PrefixUnit
deciparsec: PrefixUnit
cpc: PrefixUnit
centiparsec: PrefixUnit
mpc: PrefixUnit
milliparsec: PrefixUnit
upc: PrefixUnit
μpc: PrefixUnit
microparsec: PrefixUnit
npc: PrefixUnit
nanoparsec: PrefixUnit
ppc: PrefixUnit
picoparsec: PrefixUnit
fpc: PrefixUnit
femtoparsec: PrefixUnit
apc: PrefixUnit
attoparsec: PrefixUnit
zpc: PrefixUnit
zeptoparsec: PrefixUnit
ypc: PrefixUnit
yoctoparsec: PrefixUnit
rpc: PrefixUnit
rontoparsec: PrefixUnit
qpc: PrefixUnit
quectoparsec: PrefixUnit
solRad: Unit
R_sun: Unit
Rsun: Unit
jupiterRad: Unit
R_jup: Unit
Rjup: Unit
R_jupiter: Unit
Rjupiter: Unit
earthRad: Unit
R_earth: Unit
Rearth: Unit
lyr: Unit
lightyear: Unit
Qlyr: PrefixUnit
quettalightyear: PrefixUnit
Rlyr: PrefixUnit
ronnalightyear: PrefixUnit
Ylyr: PrefixUnit
yottalightyear: PrefixUnit
Zlyr: PrefixUnit
zettalightyear: PrefixUnit
Elyr: PrefixUnit
exalightyear: PrefixUnit
Plyr: PrefixUnit
petalightyear: PrefixUnit
Tlyr: PrefixUnit
teralightyear: PrefixUnit
Glyr: PrefixUnit
gigalightyear: PrefixUnit
Mlyr: PrefixUnit
megalightyear: PrefixUnit
klyr: PrefixUnit
kilolightyear: PrefixUnit
hlyr: PrefixUnit
hectolightyear: PrefixUnit
dalyr: PrefixUnit
dekalightyear: PrefixUnit
decalightyear: PrefixUnit
dlyr: PrefixUnit
decilightyear: PrefixUnit
clyr: PrefixUnit
centilightyear: PrefixUnit
mlyr: PrefixUnit
millilightyear: PrefixUnit
ulyr: PrefixUnit
μlyr: PrefixUnit
microlightyear: PrefixUnit
nlyr: PrefixUnit
nanolightyear: PrefixUnit
plyr: PrefixUnit
picolightyear: PrefixUnit
flyr: PrefixUnit
femtolightyear: PrefixUnit
alyr: PrefixUnit
attolightyear: PrefixUnit
zlyr: PrefixUnit
zeptolightyear: PrefixUnit
ylyr: PrefixUnit
yoctolightyear: PrefixUnit
rlyr: PrefixUnit
rontolightyear: PrefixUnit
qlyr: PrefixUnit
quectolightyear: PrefixUnit
lsec: Unit
lightsecond: Unit
solMass: Unit
M_sun: Unit
Msun: Unit
jupiterMass: Unit
M_jup: Unit
Mjup: Unit
M_jupiter: Unit
Mjupiter: Unit
earthMass: Unit
M_earth: Unit
Mearth: Unit
foe: Unit
Bethe: Unit
bethe: Unit
solLum: Unit
L_sun: Unit
Lsun: Unit
ph: IrreducibleUnit
photon: IrreducibleUnit
Qph: PrefixUnit
Qphoton: PrefixUnit
quettaphoton: PrefixUnit
Rph: PrefixUnit
Rphoton: PrefixUnit
ronnaphoton: PrefixUnit
Yph: PrefixUnit
Yphoton: PrefixUnit
yottaphoton: PrefixUnit
Zph: PrefixUnit
Zphoton: PrefixUnit
zettaphoton: PrefixUnit
Eph: PrefixUnit
Ephoton: PrefixUnit
exaphoton: PrefixUnit
Pph: PrefixUnit
Pphoton: PrefixUnit
petaphoton: PrefixUnit
Tph: PrefixUnit
Tphoton: PrefixUnit
teraphoton: PrefixUnit
Gph: PrefixUnit
Gphoton: PrefixUnit
gigaphoton: PrefixUnit
Mph: PrefixUnit
Mphoton: PrefixUnit
megaphoton: PrefixUnit
kph: PrefixUnit
kphoton: PrefixUnit
kilophoton: PrefixUnit
hph: PrefixUnit
hphoton: PrefixUnit
hectophoton: PrefixUnit
daph: PrefixUnit
daphoton: PrefixUnit
dekaphoton: PrefixUnit
decaphoton: PrefixUnit
dph: PrefixUnit
dphoton: PrefixUnit
deciphoton: PrefixUnit
cph: PrefixUnit
cphoton: PrefixUnit
centiphoton: PrefixUnit
mph: PrefixUnit
mphoton: PrefixUnit
milliphoton: PrefixUnit
uph: PrefixUnit
uphoton: PrefixUnit
μph: PrefixUnit
μphoton: PrefixUnit
microphoton: PrefixUnit
nph: PrefixUnit
nphoton: PrefixUnit
nanophoton: PrefixUnit
pph: PrefixUnit
pphoton: PrefixUnit
picophoton: PrefixUnit
fph: PrefixUnit
fphoton: PrefixUnit
femtophoton: PrefixUnit
aph: PrefixUnit
aphoton: PrefixUnit
attophoton: PrefixUnit
zph: PrefixUnit
zphoton: PrefixUnit
zeptophoton: PrefixUnit
yph: PrefixUnit
yphoton: PrefixUnit
yoctophoton: PrefixUnit
rph: PrefixUnit
rphoton: PrefixUnit
rontophoton: PrefixUnit
qph: PrefixUnit
qphoton: PrefixUnit
quectophoton: PrefixUnit
Jy: Unit
Jansky: Unit
jansky: Unit
QJy: PrefixUnit
quettaJansky: PrefixUnit
quettajansky: PrefixUnit
RJy: PrefixUnit
ronnaJansky: PrefixUnit
ronnajansky: PrefixUnit
YJy: PrefixUnit
yottaJansky: PrefixUnit
yottajansky: PrefixUnit
ZJy: PrefixUnit
zettaJansky: PrefixUnit
zettajansky: PrefixUnit
EJy: PrefixUnit
exaJansky: PrefixUnit
exajansky: PrefixUnit
PJy: PrefixUnit
petaJansky: PrefixUnit
petajansky: PrefixUnit
TJy: PrefixUnit
teraJansky: PrefixUnit
terajansky: PrefixUnit
GJy: PrefixUnit
gigaJansky: PrefixUnit
gigajansky: PrefixUnit
MJy: PrefixUnit
megaJansky: PrefixUnit
megajansky: PrefixUnit
kJy: PrefixUnit
kiloJansky: PrefixUnit
kilojansky: PrefixUnit
hJy: PrefixUnit
hectoJansky: PrefixUnit
hectojansky: PrefixUnit
daJy: PrefixUnit
dekaJansky: PrefixUnit
dekajansky: PrefixUnit
decaJansky: PrefixUnit
decajansky: PrefixUnit
dJy: PrefixUnit
deciJansky: PrefixUnit
decijansky: PrefixUnit
cJy: PrefixUnit
centiJansky: PrefixUnit
centijansky: PrefixUnit
mJy: PrefixUnit
milliJansky: PrefixUnit
millijansky: PrefixUnit
uJy: PrefixUnit
μJy: PrefixUnit
microJansky: PrefixUnit
microjansky: PrefixUnit
nJy: PrefixUnit
nanoJansky: PrefixUnit
nanojansky: PrefixUnit
pJy: PrefixUnit
picoJansky: PrefixUnit
picojansky: PrefixUnit
fJy: PrefixUnit
femtoJansky: PrefixUnit
femtojansky: PrefixUnit
aJy: PrefixUnit
attoJansky: PrefixUnit
attojansky: PrefixUnit
zJy: PrefixUnit
zeptoJansky: PrefixUnit
zeptojansky: PrefixUnit
yJy: PrefixUnit
yoctoJansky: PrefixUnit
yoctojansky: PrefixUnit
rJy: PrefixUnit
rontoJansky: PrefixUnit
rontojansky: PrefixUnit
qJy: PrefixUnit
quectoJansky: PrefixUnit
quectojansky: PrefixUnit
R: Unit
Rayleigh: Unit
rayleigh: Unit
QR: PrefixUnit
quettaRayleigh: PrefixUnit
quettarayleigh: PrefixUnit
RR: PrefixUnit
ronnaRayleigh: PrefixUnit
ronnarayleigh: PrefixUnit
YR: PrefixUnit
yottaRayleigh: PrefixUnit
yottarayleigh: PrefixUnit
ZR: PrefixUnit
zettaRayleigh: PrefixUnit
zettarayleigh: PrefixUnit
ER: PrefixUnit
exaRayleigh: PrefixUnit
exarayleigh: PrefixUnit
PR: PrefixUnit
petaRayleigh: PrefixUnit
petarayleigh: PrefixUnit
TR: PrefixUnit
teraRayleigh: PrefixUnit
terarayleigh: PrefixUnit
GR: PrefixUnit
gigaRayleigh: PrefixUnit
gigarayleigh: PrefixUnit
MR: PrefixUnit
megaRayleigh: PrefixUnit
megarayleigh: PrefixUnit
kR: PrefixUnit
kiloRayleigh: PrefixUnit
kilorayleigh: PrefixUnit
hR: PrefixUnit
hectoRayleigh: PrefixUnit
hectorayleigh: PrefixUnit
daR: PrefixUnit
dekaRayleigh: PrefixUnit
dekarayleigh: PrefixUnit
decaRayleigh: PrefixUnit
decarayleigh: PrefixUnit
dR: PrefixUnit
deciRayleigh: PrefixUnit
decirayleigh: PrefixUnit
cR: PrefixUnit
centiRayleigh: PrefixUnit
centirayleigh: PrefixUnit
mR: PrefixUnit
milliRayleigh: PrefixUnit
millirayleigh: PrefixUnit
uR: PrefixUnit
μR: PrefixUnit
microRayleigh: PrefixUnit
microrayleigh: PrefixUnit
nR: PrefixUnit
nanoRayleigh: PrefixUnit
nanorayleigh: PrefixUnit
pR: PrefixUnit
picoRayleigh: PrefixUnit
picorayleigh: PrefixUnit
fR: PrefixUnit
femtoRayleigh: PrefixUnit
femtorayleigh: PrefixUnit
aR: PrefixUnit
attoRayleigh: PrefixUnit
attorayleigh: PrefixUnit
zR: PrefixUnit
zeptoRayleigh: PrefixUnit
zeptorayleigh: PrefixUnit
yR: PrefixUnit
yoctoRayleigh: PrefixUnit
yoctorayleigh: PrefixUnit
rR: PrefixUnit
rontoRayleigh: PrefixUnit
rontorayleigh: PrefixUnit
qR: PrefixUnit
quectoRayleigh: PrefixUnit
quectorayleigh: PrefixUnit
ct: IrreducibleUnit
count: IrreducibleUnit
Qct: PrefixUnit
Qcount: PrefixUnit
quettacount: PrefixUnit
Rct: PrefixUnit
Rcount: PrefixUnit
ronnacount: PrefixUnit
Yct: PrefixUnit
Ycount: PrefixUnit
yottacount: PrefixUnit
Zct: PrefixUnit
Zcount: PrefixUnit
zettacount: PrefixUnit
Ect: PrefixUnit
Ecount: PrefixUnit
exacount: PrefixUnit
Pct: PrefixUnit
Pcount: PrefixUnit
petacount: PrefixUnit
Tct: PrefixUnit
Tcount: PrefixUnit
teracount: PrefixUnit
Gct: PrefixUnit
Gcount: PrefixUnit
gigacount: PrefixUnit
Mct: PrefixUnit
Mcount: PrefixUnit
megacount: PrefixUnit
kct: PrefixUnit
kcount: PrefixUnit
kilocount: PrefixUnit
hct: PrefixUnit
hcount: PrefixUnit
hectocount: PrefixUnit
dact: PrefixUnit
dacount: PrefixUnit
dekacount: PrefixUnit
decacount: PrefixUnit
dct: PrefixUnit
dcount: PrefixUnit
decicount: PrefixUnit
cct: PrefixUnit
ccount: PrefixUnit
centicount: PrefixUnit
mct: PrefixUnit
mcount: PrefixUnit
millicount: PrefixUnit
uct: PrefixUnit
ucount: PrefixUnit
μct: PrefixUnit
μcount: PrefixUnit
microcount: PrefixUnit
nct: PrefixUnit
ncount: PrefixUnit
nanocount: PrefixUnit
picocount: PrefixUnit
fct: PrefixUnit
fcount: PrefixUnit
femtocount: PrefixUnit
act: PrefixUnit
acount: PrefixUnit
attocount: PrefixUnit
zct: PrefixUnit
zcount: PrefixUnit
zeptocount: PrefixUnit
yct: PrefixUnit
ycount: PrefixUnit
yoctocount: PrefixUnit
rct: PrefixUnit
rcount: PrefixUnit
rontocount: PrefixUnit
qct: PrefixUnit
qcount: PrefixUnit
quectocount: PrefixUnit
adu: IrreducibleUnit
Qadu: PrefixUnit
Radu: PrefixUnit
Yadu: PrefixUnit
Zadu: PrefixUnit
Eadu: PrefixUnit
Padu: PrefixUnit
Tadu: PrefixUnit
Gadu: PrefixUnit
Madu: PrefixUnit
kadu: PrefixUnit
hadu: PrefixUnit
daadu: PrefixUnit
dadu: PrefixUnit
cadu: PrefixUnit
madu: PrefixUnit
uadu: PrefixUnit
μadu: PrefixUnit
nadu: PrefixUnit
padu: PrefixUnit
fadu: PrefixUnit
aadu: PrefixUnit
zadu: PrefixUnit
yadu: PrefixUnit
radu: PrefixUnit
qadu: PrefixUnit
DN: IrreducibleUnit
dn: IrreducibleUnit
Sun: IrreducibleUnit
chan: IrreducibleUnit
Qchan: PrefixUnit
Rchan: PrefixUnit
Ychan: PrefixUnit
Zchan: PrefixUnit
Echan: PrefixUnit
Pchan: PrefixUnit
Tchan: PrefixUnit
Gchan: PrefixUnit
Mchan: PrefixUnit
kchan: PrefixUnit
hchan: PrefixUnit
dachan: PrefixUnit
dchan: PrefixUnit
cchan: PrefixUnit
mchan: PrefixUnit
uchan: PrefixUnit
μchan: PrefixUnit
nchan: PrefixUnit
pchan: PrefixUnit
fchan: PrefixUnit
achan: PrefixUnit
zchan: PrefixUnit
ychan: PrefixUnit
rchan: PrefixUnit
qchan: PrefixUnit
bin: IrreducibleUnit
Qbin: PrefixUnit
Rbin: PrefixUnit
Ybin: PrefixUnit
Zbin: PrefixUnit
Ebin: PrefixUnit
Pbin: PrefixUnit
Tbin: PrefixUnit
Gbin: PrefixUnit
Mbin: PrefixUnit
kbin: PrefixUnit
hbin: PrefixUnit
dabin: PrefixUnit
dbin: PrefixUnit
cbin: PrefixUnit
mbin: PrefixUnit
ubin: PrefixUnit
μbin: PrefixUnit
nbin: PrefixUnit
pbin: PrefixUnit
fbin: PrefixUnit
abin: PrefixUnit
zbin: PrefixUnit
ybin: PrefixUnit
rbin: PrefixUnit
qbin: PrefixUnit
beam: IrreducibleUnit
Qbeam: PrefixUnit
Rbeam: PrefixUnit
Ybeam: PrefixUnit
Zbeam: PrefixUnit
Ebeam: PrefixUnit
Pbeam: PrefixUnit
Tbeam: PrefixUnit
Gbeam: PrefixUnit
Mbeam: PrefixUnit
kbeam: PrefixUnit
hbeam: PrefixUnit
dabeam: PrefixUnit
dbeam: PrefixUnit
cbeam: PrefixUnit
mbeam: PrefixUnit
ubeam: PrefixUnit
μbeam: PrefixUnit
nbeam: PrefixUnit
pbeam: PrefixUnit
fbeam: PrefixUnit
abeam: PrefixUnit
zbeam: PrefixUnit
ybeam: PrefixUnit
rbeam: PrefixUnit
qbeam: PrefixUnit
electron: IrreducibleUnit
